/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
* dlg_gallery.cpp
*
* Dialog for getting gallery parameters
*
* Marek Vinkler 172521@mail.muni.cz) 2009
*/

#include "v4d_basic.h"
#include "dlg_gallery.h"
#include "wdr/dlg_decor_wdr.h"

#ifdef _DEBUG
    #define new DEBUG_NEW
#endif

namespace v4d
{
	BEGIN_EVENT_TABLE(GalleryDialog, wxDialog)
		EVT_BUTTON(wxID_OK, GalleryDialog::OnOK)
	END_EVENT_TABLE();

	/*******************************************************************/

	GalleryDialog::GalleryDialog(wxWindow *parent, int imageMax):
	wxDialog(parent, wxID_ANY, "Gallery parameters")
	{
		GalleryDialogFunc(this);

		wxStaticText *label = (wxStaticText *) FindWindowById(ID_LABEL, this);
		label->SetLabel(wxString::Format("Slice count: %d", imageMax));
		wxSpinCtrl *startOffset = (wxSpinCtrl *) FindWindowById(ID_START_OFFSET, this);
		startOffset->SetRange(startOffset->GetMin(), imageMax-1);
		wxSpinCtrl *endOffset = (wxSpinCtrl *) FindWindowById(ID_END_OFFSET, this);
		endOffset->SetRange(endOffset->GetMin(), imageMax-1);
		wxSpinCtrl *step = (wxSpinCtrl *) FindWindowById(ID_STEP, this);
		step->SetRange(step->GetMin(), imageMax-1);
	}

	/*******************************************************************/

	GalleryDialog::~GalleryDialog()
	{
	}

	/*******************************************************************/

	int GalleryDialog::GetStartOffset()
	{
		wxSpinCtrl *offset = (wxSpinCtrl *) FindWindowById(ID_START_OFFSET, this);
		return offset->GetValue();
	}

	/*******************************************************************/

	int GalleryDialog::GetEndOffset()
	{
		wxSpinCtrl *offset = (wxSpinCtrl *) FindWindowById(ID_END_OFFSET, this);
		return offset->GetValue();
	}

	/*******************************************************************/

	int GalleryDialog::GetStep()
	{
		wxSpinCtrl *step = (wxSpinCtrl *) FindWindowById(ID_STEP, this);
		return step->GetValue();
	}

	/*******************************************************************/

	bool GalleryDialog::IsFixedRows()
	{
		wxRadioButton *rows = (wxRadioButton *) FindWindowById(ID_ROWS, this);
		return rows->GetValue();
	}

	/*******************************************************************/

	int GalleryDialog::GetCount()
	{
		wxSpinCtrl *count = (wxSpinCtrl *) FindWindowById(ID_COUNT, this);
		return count->GetValue();
	}

	/*******************************************************************/

	int GalleryDialog::GetSpacing()
	{
		wxSpinCtrl *spacing = (wxSpinCtrl *) FindWindowById(ID_SPACING, this);
		return spacing->GetValue();
	}

	/*******************************************************************/

	void GalleryDialog::OnOK(wxCommandEvent &evt)
	{
		wxSpinCtrl *startOffset = (wxSpinCtrl *) FindWindowById(ID_START_OFFSET, this);
		wxSpinCtrl *endOffset = (wxSpinCtrl *) FindWindowById(ID_END_OFFSET, this);

		if(startOffset->GetValue() > endOffset->GetValue())
		{
			wxMessageBox("'Starting slice' must be less or equal to 'Ending slice'",
				"Bad input", wxICON_EXCLAMATION, this);
			return;
		}

		EndDialog(wxID_OK);
	}
}
