/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * dlg_pref.cpp
 *
 * Implementation of dialog for changing screen options
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006
 */

#include "v4d_basic.h"
#include <wx/colordlg.h>
#include <wx/fontdlg.h>
#include "dlg_pref.h"
#include "wdr/preferences_wdr.h"
#include "wxtools.h"

#ifdef _DEBUG
    #define new DEBUG_NEW
#endif

namespace v4d 
{
	BEGIN_EVENT_TABLE(PreferencesDialog,wxDialog)
		EVT_RADIOBOX(ID_PREF_POINTER_TYPE, PreferencesDialog::OnChangePointerStyle)
		EVT_RADIOBOX(ID_PREF_XYANN_CORNER, PreferencesDialog::OnChangeXyAnnotPosition)
		EVT_RADIOBOX(ID_PREF_XZANN_CORNER, PreferencesDialog::OnChangeXzAnnotPosition)
		EVT_RADIOBOX(ID_PREF_YZANN_CORNER, PreferencesDialog::OnChangeYzAnnotPosition)
		EVT_BUTTON(ID_PREF_POINTER_COLOUR, PreferencesDialog::OnChangePointerColour)
		EVT_BUTTON(ID_PREF_ANN_FONT, PreferencesDialog::OnChangeAnnotationFont)
		EVT_BUTTON(ID_PREF_ANN_COLOUR, PreferencesDialog::OnChangeAnnotationColour)
		EVT_BUTTON(ID_PREF_SB_COLOUR, PreferencesDialog::OnChangeScaleBarColour)
		EVT_BUTTON(ID_PREF_TB_COLOUR, PreferencesDialog::OnChangeTimeBarColour)
		EVT_RADIOBOX(ID_PREF_SB_CORNER, PreferencesDialog::OnChangeScaleBarCorner)
		EVT_RADIOBOX(ID_PREF_TB_CORNER, PreferencesDialog::OnChangeTimeBarCorner)
	END_EVENT_TABLE()

    /*******************************************************************/

	PreferencesDialog::PreferencesDialog(wxWindow *parent, v4d_CanvasSettings *settings, wxColourData *palette):
		wxDialog(parent, -1, "Screen options dialog", wxDefaultPosition, wxDefaultSize), m_settings(settings),
        m_palette(palette)
	{
		SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
		PreferencesDialogFunc(this);

		m_res_x_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RES_X);
		m_res_x_ctrl -> SetValidator(FloatValidator(m_settings -> orig_res.x));
		m_res_x_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_res_y_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RES_Y);
		m_res_y_ctrl -> SetValidator(FloatValidator(m_settings -> orig_res.y));
		m_res_y_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_res_z_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RES_Z);
		m_res_z_ctrl -> SetValidator(FloatValidator(m_settings -> orig_res.z));
		m_res_z_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_rat_x_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RAT_X);
		m_rat_x_ctrl -> SetValue(wxString::Format("%f", 1.0));
		m_rat_x_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_rat_y_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RAT_Y);
		m_rat_y_ctrl -> SetValue(wxString::Format("%f", m_settings -> orig_res.y / m_settings -> orig_res.x));
		m_rat_y_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_rat_z_ctrl = (wxTextCtrl *) FindWindow(ID_PREF_RAT_Z);
		m_rat_z_ctrl -> SetValue(wxString::Format("%f", m_settings -> orig_res.z / m_settings -> orig_res.x));
		m_rat_z_ctrl -> Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);

		m_cut_ctrl = (wxRadioBox *) FindWindow(ID_PREF_POINTER_TYPE);
		m_cut_ctrl -> SetSelection(m_settings -> cut_pointer_style.type);

		m_cut_bitmap =  (wxStaticBitmap *) FindWindow(ID_PREF_POINTER_COLOUR);
		SetBitmapColour();

		//Projection annotation

		m_anot_xy_text = (wxTextCtrl *)FindWindow(ID_PREF_XYANN_TEXT);
		m_anot_xy_text->SetValue(m_settings->projection_annotation_style.xyText);
		m_anot_xy_text->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_anot_xz_text = (wxTextCtrl *)FindWindow(ID_PREF_XZANN_TEXT);
		m_anot_xz_text->SetValue(m_settings->projection_annotation_style.xzText);
		m_anot_xz_text->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_anot_yz_text = (wxTextCtrl *)FindWindow(ID_PREF_YZANN_TEXT);
		m_anot_yz_text->SetValue(m_settings->projection_annotation_style.yzText);
		m_anot_yz_text->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		
		m_anot_xypos_radiobox = (wxRadioBox *)FindWindow(ID_PREF_XYANN_CORNER);
		m_anot_xypos_radiobox->SetSelection(m_settings->projection_annotation_style.xyTextPos);
		m_anot_xzpos_radiobox = (wxRadioBox *)FindWindow(ID_PREF_XZANN_CORNER);
		m_anot_xzpos_radiobox->SetSelection(m_settings->projection_annotation_style.xzTextPos);
		m_anot_yzpos_radiobox = (wxRadioBox *)FindWindow(ID_PREF_YZANN_CORNER);
		m_anot_yzpos_radiobox->SetSelection(m_settings->projection_annotation_style.yzTextPos);

		m_pref_xyann_offset_x = (wxSpinCtrl *)FindWindow(ID_PREF_XYANN_OFFSET_X);
		m_pref_xyann_offset_x->SetValue(m_settings->projection_annotation_style.xyOffset.x);
		m_pref_xyann_offset_x->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xyann_offset_y = (wxSpinCtrl *)FindWindow(ID_PREF_XYANN_OFFSET_Y);
		m_pref_xyann_offset_y->SetValue(m_settings->projection_annotation_style.xyOffset.y);
		m_pref_xyann_offset_y->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xzann_offset_x = (wxSpinCtrl *)FindWindow(ID_PREF_XZANN_OFFSET_X);
		m_pref_xzann_offset_x->SetValue(m_settings->projection_annotation_style.xzOffset.x);
		m_pref_xzann_offset_x->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xzann_offset_y = (wxSpinCtrl *)FindWindow(ID_PREF_XZANN_OFFSET_Y);
		m_pref_xzann_offset_y->SetValue(m_settings->projection_annotation_style.xzOffset.y);
		m_pref_xzann_offset_y->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_yzann_offset_x = (wxSpinCtrl *)FindWindow(ID_PREF_YZANN_OFFSET_X);
		m_pref_yzann_offset_x->SetValue(m_settings->projection_annotation_style.yzOffset.x);
		m_pref_yzann_offset_x->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_yzann_offset_y = (wxSpinCtrl *)FindWindow(ID_PREF_YZANN_OFFSET_Y);
		m_pref_yzann_offset_y->SetValue(m_settings->projection_annotation_style.yzOffset.y);
		m_pref_yzann_offset_y->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
	
		//Scale bar

		m_pref_sblength_value = (wxTextCtrl *)FindWindow(ID_PREF_SBLENGTH_VALUE);
		m_pref_sblength_value->SetValidator(FloatValidator(m_settings->scale_bar_style.length,0.f,1000000.f));
		m_pref_sbwidth_value = (wxSpinCtrl *)FindWindow(ID_PREF_SBWIDTH_VALUE);
		m_pref_sbwidth_value->SetValue(m_settings->scale_bar_style.width);
		m_pref_sbwidth_value->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_sb_corner = (wxRadioBox *)FindWindow(ID_PREF_SB_CORNER);
		m_pref_sb_corner->SetSelection(m_settings->scale_bar_style.corner);
		m_pref_sb_offset_x = (wxSpinCtrl*)FindWindow(ID_PREF_SB_OFFSET_X);
		m_pref_sb_offset_x->SetValue(m_settings->scale_bar_style.offset.x);
		m_pref_sb_offset_x->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_sb_offset_y = (wxSpinCtrl*)FindWindow(ID_PREF_SB_OFFSET_Y);
		m_pref_sb_offset_y->SetValue(m_settings->scale_bar_style.offset.y);
		m_pref_sb_offset_y->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);

		//Time bar

		m_pref_tblength_value = (wxSpinCtrl *)FindWindow(ID_PREF_TBLENGTH_VALUE);
		m_pref_tblength_value->SetValue(m_settings->time_bar_style.length);
		m_pref_tblength_value->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tbwidth_value = (wxSpinCtrl *)FindWindow(ID_PREF_TBWIDTH_VALUE);
		m_pref_tbwidth_value->SetValue(m_settings->time_bar_style.width);
		m_pref_tbwidth_value->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tb_corner = (wxRadioBox *)FindWindow(ID_PREF_TB_CORNER);
		m_pref_tb_corner->SetSelection(m_settings->time_bar_style.corner);
		m_pref_tb_offset_x = (wxSpinCtrl*)FindWindow(ID_PREF_TB_OFFSET_X);
		m_pref_tb_offset_x->SetValue(m_settings->time_bar_style.offset.x);
		m_pref_tb_offset_x->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tb_offset_y = (wxSpinCtrl*)FindWindow(ID_PREF_TB_OFFSET_Y);
		m_pref_tb_offset_y->SetValue(m_settings->time_bar_style.offset.y);
		m_pref_tb_offset_y->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
	}


	PreferencesDialog::~PreferencesDialog()
	{
		//at least, disconnect all dynamically connected event handlers (thought some destructor upstream might do that as well...)
		m_res_x_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_res_y_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_res_z_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_rat_x_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_rat_y_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_rat_z_ctrl -> Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_anot_xy_text->Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_anot_xz_text->Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_anot_yz_text->Disconnect(wxEVT_KILL_FOCUS, wxFocusEventHandler(PreferencesDialog::OnKillFocus), NULL, this);
		m_pref_xyann_offset_x->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xyann_offset_y->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xzann_offset_x->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_xzann_offset_y->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_yzann_offset_x->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_yzann_offset_y->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_sbwidth_value->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_sb_offset_x->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_sb_offset_y->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tblength_value->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tbwidth_value->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tb_offset_x->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
		m_pref_tb_offset_y->Disconnect(wxEVT_SPINCTRL, wxSpinEventHandler(PreferencesDialog::OnSpinEvent), NULL, this);
	}

    /*******************************************************************/

	void PreferencesDialog::OnKillFocus(wxFocusEvent &event)
	{
		switch(event.GetId())
		{
			case ID_PREF_RES_X: UpdateAspectRatioControl(m_res_x_ctrl, m_rat_x_ctrl); break;
			case ID_PREF_RES_Y: UpdateAspectRatioControl(m_res_y_ctrl, m_rat_y_ctrl); break;
			case ID_PREF_RES_Z: UpdateAspectRatioControl(m_res_z_ctrl, m_rat_z_ctrl); break;
			case ID_PREF_RAT_X: UpdateResolutionControl(m_rat_x_ctrl, m_res_x_ctrl); break; 
			case ID_PREF_RAT_Y: UpdateResolutionControl(m_rat_y_ctrl, m_res_y_ctrl); break; 
			case ID_PREF_RAT_Z: UpdateResolutionControl(m_rat_z_ctrl, m_res_z_ctrl); break;
			case ID_PREF_XYANN_TEXT: m_settings->projection_annotation_style.xyText = m_anot_xy_text->GetValue(); break;
			case ID_PREF_XZANN_TEXT: m_settings->projection_annotation_style.xzText = m_anot_xz_text->GetValue(); break;
			case ID_PREF_YZANN_TEXT: m_settings->projection_annotation_style.yzText = m_anot_yz_text->GetValue(); break;
		}
	}

	/*******************************************************************/

	void PreferencesDialog::OnSpinEvent(wxSpinEvent &event)
	{
		switch (event.GetId())
		{
			case ID_PREF_XYANN_OFFSET_X:
				m_settings->projection_annotation_style.xyOffset.x = m_pref_xyann_offset_x->GetValue();
				break;
			case ID_PREF_XYANN_OFFSET_Y:
				m_settings->projection_annotation_style.xyOffset.y = m_pref_xyann_offset_y->GetValue();
				break;
			case ID_PREF_XZANN_OFFSET_X:
				m_settings->projection_annotation_style.xzOffset.x = m_pref_xzann_offset_x->GetValue();
				break;
			case ID_PREF_XZANN_OFFSET_Y:
				m_settings->projection_annotation_style.xzOffset.y = m_pref_xzann_offset_y->GetValue();
				break;
			case ID_PREF_YZANN_OFFSET_X:
				m_settings->projection_annotation_style.yzOffset.x = m_pref_yzann_offset_x->GetValue();
				break;
			case ID_PREF_YZANN_OFFSET_Y:
				m_settings->projection_annotation_style.yzOffset.y = m_pref_yzann_offset_y->GetValue();
				break;
			case ID_PREF_SBWIDTH_VALUE:
				m_settings->scale_bar_style.width = m_pref_sbwidth_value->GetValue();
				break;
			case ID_PREF_SB_OFFSET_X:
				m_settings->scale_bar_style.offset.x = m_pref_sb_offset_x->GetValue();
				break;
			case ID_PREF_SB_OFFSET_Y:
				m_settings->scale_bar_style.offset.y = m_pref_sb_offset_y->GetValue();
				break;
			case ID_PREF_TBLENGTH_VALUE:
				m_settings->time_bar_style.length = m_pref_tblength_value->GetValue();
				break;
			case ID_PREF_TBWIDTH_VALUE:
				m_settings->time_bar_style.width = m_pref_tbwidth_value->GetValue();
				break;
			case ID_PREF_TB_OFFSET_X:
				m_settings->time_bar_style.offset.x = m_pref_tb_offset_x->GetValue();
				break;
			case ID_PREF_TB_OFFSET_Y:
				m_settings->time_bar_style.offset.y = m_pref_tb_offset_y->GetValue();
				break;
		}
	}

    /*******************************************************************/

	void PreferencesDialog::OnChangePointerStyle(wxCommandEvent &event)
	{
		m_settings -> cut_pointer_style.type = (v4d_CutPointerType) m_cut_ctrl -> GetSelection();
	}

    /*******************************************************************/

	void PreferencesDialog::OnChangePointerColour(wxCommandEvent &event)
	{
		m_palette -> SetChooseFull(true);
		m_palette -> SetColour(m_settings -> cut_pointer_style.colour);

		wxColourDialog dlg(this, m_palette);

		if (dlg.ShowModal() == wxID_OK)
		{
			*m_palette = dlg.GetColourData();
			m_settings -> cut_pointer_style.colour = m_palette -> GetColour();
			SetBitmapColour();
 	    }
	}

    /*******************************************************************/

	void PreferencesDialog::SetBitmapColour()
	{
		m_cut_bitmap -> SetBackgroundColour(m_settings -> cut_pointer_style.colour);
		m_cut_bitmap -> SetForegroundColour(m_settings -> cut_pointer_style.colour);
		m_cut_bitmap -> ClearBackground();
	}

    /*******************************************************************/

	void PreferencesDialog::UpdateAspectRatioControl(const wxTextCtrl *res_ctrl, wxTextCtrl *ratio_ctrl)
	{
		double val;

        if (res_ctrl -> IsModified() && res_ctrl -> GetValue().ToDouble(&val))
			ratio_ctrl -> SetValue(wxString::Format("%f", val / m_settings -> orig_res.x));
	}

    /*******************************************************************/

	void PreferencesDialog::UpdateResolutionControl(const wxTextCtrl *ratio_ctrl, wxTextCtrl *res_ctrl)
	{
		double val;

		if (ratio_ctrl -> IsModified() && ratio_ctrl -> GetValue().ToDouble(&val))
			res_ctrl -> SetValue(wxString::Format("%f", val * m_settings -> orig_res.x));
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeAnnotationFont(wxCommandEvent &event)
	{
		wxFontData initialFont;
		initialFont.SetColour(m_settings->projection_annotation_style.colour);
		initialFont.SetInitialFont(m_settings->projection_annotation_style.font);

		//create the dialog suggesting it show the last used font (and colour)
		wxFontDialog dlg(this,initialFont);

		if (dlg.ShowModal() == wxID_OK)
		{
			m_settings->projection_annotation_style.colour = dlg.GetFontData().GetColour();
			m_settings->projection_annotation_style.font = dlg.GetFontData().GetChosenFont();
		}
	}

    /*******************************************************************/

	void PreferencesDialog::OnChangeAnnotationColour(wxCommandEvent &event)
	{
		wxColourDialog dlg(this);
		dlg.GetColourData().SetColour(m_settings->projection_annotation_style.colour);

		if (dlg.ShowModal() == wxID_OK)
		{
			m_settings->projection_annotation_style.colour = dlg.GetColourData().GetColour();
		}
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeXyAnnotPosition(wxCommandEvent &event)
	{
		m_settings->projection_annotation_style.xyTextPos = (v4d_TextPosition)m_anot_xypos_radiobox->GetSelection();
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeXzAnnotPosition(wxCommandEvent &event)
	{
		m_settings->projection_annotation_style.xzTextPos = (v4d_TextPosition)m_anot_xzpos_radiobox->GetSelection();
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeYzAnnotPosition(wxCommandEvent &event)
	{
		m_settings->projection_annotation_style.yzTextPos = (v4d_TextPosition)m_anot_yzpos_radiobox->GetSelection();
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeScaleBarColour(wxCommandEvent &event)
	{
		wxColourDialog dlg(this);
		dlg.GetColourData().SetColour(m_settings->scale_bar_style.colour);

		if (dlg.ShowModal() == wxID_OK)
		{
			m_settings->scale_bar_style.colour = dlg.GetColourData().GetColour();
		}
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeTimeBarColour(wxCommandEvent &event)
	{
		wxColourDialog dlg(this);
		dlg.GetColourData().SetChooseFull(true);
		dlg.GetColourData().SetColour(m_settings->time_bar_style.colour);

		if (dlg.ShowModal() == wxID_OK)
		{
			m_settings->time_bar_style.colour = dlg.GetColourData().GetColour();
		}
	}


	/*******************************************************************/

	void PreferencesDialog::OnChangeScaleBarCorner(wxCommandEvent &event)
	{
		m_settings->scale_bar_style.corner = (v4d_TextPosition)m_pref_sb_corner->GetSelection();
	}

	/*******************************************************************/

	void PreferencesDialog::OnChangeTimeBarCorner(wxCommandEvent &event)
	{
		m_settings->time_bar_style.corner = (v4d_TextPosition)m_pref_tb_corner->GetSelection();
	}
}

