/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * dlg_set_range.h
 *
 * Implementation of set range dialog
 *
 * Pavel Matula (pam@fi.muni.cz) 2007
 *
 */

#include "v4d_basic.h"
#include <wx/dialog.h>
#include "wxtools.h"
#include "dlg_set_range.h"
#include "wdr/dlg_range_wdr.h"

#ifdef _DEBUG
    #define new DEBUG_NEW
#endif

namespace v4d 
{
    /************************************************************************************************/

	class SetFloatRangeDlg : public wxDialog {
		wxTextCtrl *_min_value_ctrl;
		wxTextCtrl *_max_value_ctrl;
    public: 
		SetFloatRangeDlg(wxWindow *parent, float min_limit, float max_limit, float &min_value, float &max_value);
		void OnOK(wxCommandEvent &key);
		DECLARE_EVENT_TABLE()
	};


    BEGIN_EVENT_TABLE(SetFloatRangeDlg, wxDialog)
	    EVT_BUTTON(wxID_OK, SetFloatRangeDlg::OnOK)
    END_EVENT_TABLE()

    /************************************************************************************************/

	SetFloatRangeDlg::SetFloatRangeDlg(wxWindow *parent, float min_limit, float max_limit, float &min_value, float &max_value)
		: wxDialog(parent, -1, "Set Range", wxDefaultPosition, wxDefaultSize, 
	       wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
	{
        wxASSERT (min_limit < max_limit);

		SetRangeDialogFunc(this);
		wxStaticText * min_limit_text = (wxStaticText *) FindWindow(ID_MINLIMIT);
		wxStaticText * max_limit_text = (wxStaticText *) FindWindow(ID_MAXLIMIT);
		_min_value_ctrl = (wxTextCtrl *) FindWindow(ID_MINVALUE);
		_max_value_ctrl = (wxTextCtrl *) FindWindow(ID_MAXVALUE);
        
		wxASSERT (min_limit_text);
        wxASSERT (max_limit_text);
		wxASSERT (_min_value_ctrl);
		wxASSERT (_max_value_ctrl);

		min_limit_text->SetLabel(wxString::Format("%f", min_limit));
		max_limit_text->SetLabel(wxString::Format("%f", max_limit));

        // Setup validators
        _min_value_ctrl->SetValidator(FloatValidator(min_value, min_limit, max_limit));
        _max_value_ctrl->SetValidator(FloatValidator(max_value, min_limit, max_limit));
	}

    /************************************************************************************************/
		
	void SetFloatRangeDlg::OnOK(wxCommandEvent &key)
	{
		double a, b;
		if (Validate()) {
			if (_min_value_ctrl->GetValue().ToDouble(&a) && _max_value_ctrl->GetValue().ToDouble(&b) && a < b) {
				if (TransferDataFromWindow()) {
					if (IsModal()) {
						EndModal(wxID_OK);
					} else {
						SetReturnCode(wxID_OK);
						Show(false);
					}
				}
			} else {
				wxMessageBox("Min value must be less than max value.");
			}
		}
	}

    /************************************************************************************************/

	class SetLongRangeDlg : public wxDialog {
		wxTextCtrl *_min_value_ctrl;
		wxTextCtrl *_max_value_ctrl;
    public: 
		SetLongRangeDlg(wxWindow *parent, long min_limit, long max_limit, long &min_value, long &max_value);
		void OnOK(wxCommandEvent &key);
		DECLARE_EVENT_TABLE()
	};


    BEGIN_EVENT_TABLE(SetLongRangeDlg, wxDialog)
        EVT_BUTTON(wxID_OK, SetLongRangeDlg::OnOK)
    END_EVENT_TABLE()

    /************************************************************************************************/

	SetLongRangeDlg::SetLongRangeDlg(wxWindow *parent, long min_limit, long max_limit, long &min_value, long &max_value)
		: wxDialog(parent, -1, "Set Range", wxDefaultPosition, wxDefaultSize, 
	       wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
	{
        wxASSERT (min_limit < max_limit);

		SetRangeDialogFunc(this);
		wxStaticText * min_limit_text = (wxStaticText *) FindWindow(ID_MINLIMIT);
		wxStaticText * max_limit_text = (wxStaticText *) FindWindow(ID_MAXLIMIT);
		_min_value_ctrl = (wxTextCtrl *) FindWindow(ID_MINVALUE);
		_max_value_ctrl = (wxTextCtrl *) FindWindow(ID_MAXVALUE);
        
		wxASSERT (min_limit_text);
        wxASSERT (max_limit_text);
		wxASSERT (_min_value_ctrl);
		wxASSERT (_max_value_ctrl);

		min_limit_text->SetLabel(wxString::Format("%ld", min_limit));
		max_limit_text->SetLabel(wxString::Format("%ld", max_limit));

        // Setup validators
        _min_value_ctrl->SetValidator(LongValidator (min_value, min_limit, max_limit));
        _max_value_ctrl->SetValidator(LongValidator  (max_value, min_limit, max_limit));
	}

    /************************************************************************************************/
		
	void SetLongRangeDlg::OnOK(wxCommandEvent &key)
	{
		long a, b;
		if (Validate()) {
			if (_min_value_ctrl->GetValue().ToLong(&a) && _max_value_ctrl->GetValue().ToLong(&b) && a < b) {
				if (TransferDataFromWindow()) {
					if (IsModal()) {
						EndModal(wxID_OK);
					} else {
						SetReturnCode(wxID_OK);
						Show(false);
					}
				}
			} else {
				wxMessageBox("Min value must be less than max value.");
			}
		}
	}

    /************************************************************************************************/

	template <class T> class SetRGBRangeDlg : public wxDialog 
	{
	  private:
		wxTextCtrl *_red_min_value_ctrl;
		wxTextCtrl *_red_max_value_ctrl;
		wxTextCtrl *_green_min_value_ctrl;
		wxTextCtrl *_green_max_value_ctrl;
		wxTextCtrl *_blue_min_value_ctrl;
		wxTextCtrl *_blue_max_value_ctrl;
		wxButton *_red_apply_button;
		wxButton *_green_apply_button;
		wxButton *_blue_apply_button;

		i3d::RGB_generic<T> _min_limit;
		i3d::RGB_generic<T> _max_limit;

	
	  public: 
        /** Constructor. */ 
		SetRGBRangeDlg(wxWindow *parent, const i3d::RGB_generic<T> &min_limit, const i3d::RGB_generic<T> &max_limit, 
			           i3d::RGB_generic<T> &min_value, i3d::RGB_generic<T> &max_value);

	  private:
        /** Event handler for the OK button. */
		void OnOK(wxCommandEvent &event);
		/** Event handler for Apply buttons. */
		void OnApplyButton(wxCommandEvent &event);
		/** Event handler for update UI events. */
		void OnUpdateUI(wxUpdateUIEvent &event);
        /** Return true if the given range is also valid the given channel. */
		bool IsRangeValid(long min, long max, int channel);
	};

    /************************************************************************************************/

	template <class T> SetRGBRangeDlg<T>::SetRGBRangeDlg(wxWindow *parent, const i3d::RGB_generic<T> &min_limit, 
		const i3d::RGB_generic<T> &max_limit, i3d::RGB_generic<T> &min_value, i3d::RGB_generic<T> &max_value)
		: wxDialog(parent, -1, "Set Range", wxDefaultPosition, wxDefaultSize, 
		wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER), _min_limit(min_limit), _max_limit(max_limit)
	{
        wxASSERT(min_limit < max_limit);

		SetRGBRangeDialogFunc(this);

		// find controls
		wxStaticText *red_min_limit_text = (wxStaticText *) FindWindow(ID_RED_MINLIMIT);
		wxASSERT(red_min_limit_text);
		red_min_limit_text -> SetLabel(wxString::Format("%d", min_limit.red));

		wxStaticText *red_max_limit_text = (wxStaticText *) FindWindow(ID_RED_MAXLIMIT);
		wxASSERT(red_max_limit_text);
		red_max_limit_text -> SetLabel(wxString::Format("%d", max_limit.red));

		wxStaticText *green_min_limit_text = (wxStaticText *) FindWindow(ID_GREEN_MINLIMIT);
		wxASSERT(green_min_limit_text);
		green_min_limit_text -> SetLabel(wxString::Format("%d", min_limit.green));

		wxStaticText *green_max_limit_text = (wxStaticText *) FindWindow(ID_GREEN_MAXLIMIT);
		wxASSERT(green_max_limit_text);
		green_max_limit_text -> SetLabel(wxString::Format("%d", max_limit.green));

		wxStaticText *blue_min_limit_text = (wxStaticText *) FindWindow(ID_BLUE_MINLIMIT);
		wxASSERT(blue_min_limit_text);
		blue_min_limit_text -> SetLabel(wxString::Format("%d", min_limit.blue));

		wxStaticText *blue_max_limit_text = (wxStaticText *) FindWindow(ID_BLUE_MAXLIMIT);
		wxASSERT(blue_max_limit_text);
		blue_max_limit_text -> SetLabel(wxString::Format("%d", max_limit.blue));

		_red_min_value_ctrl = (wxTextCtrl *) FindWindow(ID_RED_MINVALUE);
		wxASSERT(_red_min_value_ctrl);
		_red_min_value_ctrl -> SetValidator(IntValidator<T>(min_value.red, min_limit.red, max_limit.red));

		_red_max_value_ctrl = (wxTextCtrl *) FindWindow(ID_RED_MAXVALUE);
		wxASSERT(_red_max_value_ctrl);
  	    _red_max_value_ctrl -> SetValidator(IntValidator<T>(max_value.red, min_limit.red, max_limit.red));

		_green_min_value_ctrl = (wxTextCtrl *) FindWindow(ID_GREEN_MINVALUE);
		wxASSERT(_green_min_value_ctrl);
		_green_min_value_ctrl -> SetValidator(IntValidator<T>(min_value.green, min_limit.green, max_limit.green));

		_green_max_value_ctrl = (wxTextCtrl *) FindWindow(ID_GREEN_MAXVALUE);
		wxASSERT(_green_max_value_ctrl);
  	    _green_max_value_ctrl -> SetValidator(IntValidator<T>(max_value.green, min_limit.green, max_limit.green));

		_blue_min_value_ctrl = (wxTextCtrl *) FindWindow(ID_BLUE_MINVALUE);
		wxASSERT(_blue_min_value_ctrl);
		_blue_min_value_ctrl -> SetValidator(IntValidator<T>(min_value.blue, min_limit.blue, max_limit.blue));

		_blue_max_value_ctrl = (wxTextCtrl *) FindWindow(ID_BLUE_MAXVALUE);
		wxASSERT(_blue_max_value_ctrl);
  	    _blue_max_value_ctrl -> SetValidator(IntValidator<T>(max_value.blue, min_limit.blue, max_limit.blue));

		_red_apply_button = (wxButton *) FindWindow(ID_RED_APPLY_BUTTON);
		wxASSERT(_red_apply_button);

		_green_apply_button = (wxButton *) FindWindow(ID_GREEN_APPLY_BUTTON);
		wxASSERT(_green_apply_button);

		_blue_apply_button = (wxButton *) FindWindow(ID_BLUE_APPLY_BUTTON);
		wxASSERT(_blue_apply_button);

		// connect handler for OK button
        Bind(wxEVT_COMMAND_BUTTON_CLICKED, &SetRGBRangeDlg<T>::OnOK, this, wxID_OK);

		// connect handler for Apply buttons
        Bind(wxEVT_COMMAND_BUTTON_CLICKED, &SetRGBRangeDlg<T>::OnApplyButton, this, ID_RED_APPLY_BUTTON);
        Bind(wxEVT_COMMAND_BUTTON_CLICKED, &SetRGBRangeDlg<T>::OnApplyButton, this, ID_GREEN_APPLY_BUTTON);
        Bind(wxEVT_COMMAND_BUTTON_CLICKED, &SetRGBRangeDlg<T>::OnApplyButton, this, ID_BLUE_APPLY_BUTTON);

		// connect handler for update UI events
        Bind(wxEVT_UPDATE_UI, &SetRGBRangeDlg<T>::OnUpdateUI, this, ID_RED_APPLY_BUTTON);
        Bind(wxEVT_UPDATE_UI, &SetRGBRangeDlg<T>::OnUpdateUI, this, ID_GREEN_APPLY_BUTTON);
        Bind(wxEVT_UPDATE_UI, &SetRGBRangeDlg<T>::OnUpdateUI, this, ID_BLUE_APPLY_BUTTON);
	}

    /************************************************************************************************/
		
	template <class T> void SetRGBRangeDlg<T>::OnOK(wxCommandEvent &event)
	{
		long red_min, red_max, green_min, green_max, blue_min, blue_max;
		if (Validate()) {
			if (_red_min_value_ctrl->GetValue().ToLong(&red_min) && _red_max_value_ctrl->GetValue().ToLong(&red_max) && red_min < red_max &&
                _green_min_value_ctrl->GetValue().ToLong(&green_min) && _green_max_value_ctrl->GetValue().ToLong(&green_max) && green_min < green_max &&
				_blue_min_value_ctrl->GetValue().ToLong(&blue_min) && _blue_max_value_ctrl->GetValue().ToLong(&blue_max) && blue_min < blue_max)
			{
				if (TransferDataFromWindow()) {
					if (IsModal()) {
						EndModal(wxID_OK);
					} else {
						SetReturnCode(wxID_OK);
						Show(false);
					}
				}
			} else {
				wxMessageBox("Min value must be less than max value.");
			}
		}
	}

    /************************************************************************************************/

	template <class T> void SetRGBRangeDlg<T>::OnApplyButton(wxCommandEvent &event)
	{
		switch (event.GetId())
		{
		  case ID_RED_APPLY_BUTTON:
			  _green_min_value_ctrl->SetValue(_red_min_value_ctrl->GetValue());
			  _blue_min_value_ctrl->SetValue(_red_min_value_ctrl->GetValue());
			  _green_max_value_ctrl->SetValue(_red_max_value_ctrl->GetValue());
			  _blue_max_value_ctrl->SetValue(_red_max_value_ctrl->GetValue());
			  break;

		  case ID_GREEN_APPLY_BUTTON:
			  _red_min_value_ctrl->SetValue(_green_min_value_ctrl->GetValue());
			  _blue_min_value_ctrl->SetValue(_green_min_value_ctrl->GetValue());
			  _red_max_value_ctrl->SetValue(_green_max_value_ctrl->GetValue());
			  _blue_max_value_ctrl->SetValue(_green_max_value_ctrl->GetValue());
			  break;

		  case ID_BLUE_APPLY_BUTTON:
			  _red_min_value_ctrl->SetValue(_blue_min_value_ctrl->GetValue());
			  _green_min_value_ctrl->SetValue(_blue_min_value_ctrl->GetValue());
			  _red_max_value_ctrl->SetValue(_blue_max_value_ctrl->GetValue());
			  _green_max_value_ctrl->SetValue(_blue_max_value_ctrl->GetValue());
			  break;
		}
	}

    /************************************************************************************************/

	template <class T> void SetRGBRangeDlg<T>::OnUpdateUI(wxUpdateUIEvent &event)
	{
		long min, max;

		switch (event.GetId())
		{
			case ID_RED_APPLY_BUTTON: 
				event.Enable(_red_min_value_ctrl->GetValue().ToLong(&min) &&
					         _red_max_value_ctrl->GetValue().ToLong(&max) &&  
							 IsRangeValid(min, max, 1) && IsRangeValid(min, max, 2));
				break;

			case ID_GREEN_APPLY_BUTTON:
				event.Enable(_green_min_value_ctrl->GetValue().ToLong(&min) &&
					         _green_max_value_ctrl->GetValue().ToLong(&max) &&  
							 IsRangeValid(min, max, 0) && IsRangeValid(min, max, 2));
				break;

			case ID_BLUE_APPLY_BUTTON:
				event.Enable(_blue_min_value_ctrl->GetValue().ToLong(&min) &&
					         _blue_max_value_ctrl->GetValue().ToLong(&max) &&  
							 IsRangeValid(min, max, 0) && IsRangeValid(min, max, 1));
				break;
		}
	}

    /************************************************************************************************/

	template <class T> bool SetRGBRangeDlg<T>::IsRangeValid(long min, long max, int channel)
	{
		return (min <= max && min >= static_cast<long>(_min_limit.GetComponent(channel)) && 
			    max <= static_cast<long>(_max_limit.GetComponent(channel)));
	}

    /************************************************************************************************/
    /************************************************************************************************/

	bool GetLongRangeFromUser(wxWindow *parent, long min_limit, long max_limit, long &min_value, long &max_value)
	{
		SetLongRangeDlg dlg(parent, min_limit, max_limit, min_value, max_value);

		if (dlg.ShowModal() == wxID_OK) {
			return true;
		}

		return false;
	}

    /************************************************************************************************/

	bool GetFloatRangeFromUser(wxWindow *parent, float min_limit, float max_limit, float &min_value, float &max_value)
	{
		SetFloatRangeDlg dlg(parent, min_limit, max_limit, min_value, max_value);

		if (dlg.ShowModal() == wxID_OK) {
			return true;
		}

		return false;
	}

    /************************************************************************************************/

	template <class T> bool GetRGBRangeFromUser(wxWindow *parent, i3d::RGB_generic<T> min_limit, 
		i3d::RGB_generic<T> max_limit, i3d::RGB_generic<T> &min_value, i3d::RGB_generic<T> &max_value)
	{
		SetRGBRangeDlg<T> dlg(parent, min_limit, max_limit, min_value, max_value);

		return (dlg.ShowModal() == wxID_OK);
	}

    /************************************************************************************************/

	template class SetRGBRangeDlg<i3d::GRAY8>;
	template class SetRGBRangeDlg<i3d::GRAY16>;
	template bool GetRGBRangeFromUser(wxWindow *parent, i3d::RGB min_limit, i3d::RGB max_limit, 
									  i3d::RGB &min_value, i3d::RGB &max_value);
	template bool GetRGBRangeFromUser(wxWindow *parent, i3d::RGB16 min_limit, i3d::RGB16 max_limit, 
			 						  i3d::RGB16 &min_value, i3d::RGB16 &max_value);

    /************************************************************************************************/
} // namespace v4d


