/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * image_cropper.h
 *
 * This file defines ImageCropper class
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006
 */

#ifndef V4D_IMAGE_CROPPER_H
#define V4D_IMAGE_CROPPER_H

#include "types/box.h"

namespace bcrop 
{	
	/** The ImageCropper class is a base abstract class that provides functions for loading and saving 
	boxes and cropping images. */
	template <class T> class ImageCropper
	{
	  public:
		/** Constructor. */
		ImageCropper() {};
		/** Destructor. */
		virtual ~ImageCropper() {}; 
		/** Load boxes for all positions from output directory. Function returns true if loading was successful. */
		virtual bool LoadAllBoxes(const wxString &odir, v4d::MapOfPositionBoxes &mp) = 0;
		/** Save boxes for all positions to output directory. Function returns true if saving was successful. */
		virtual bool SaveAllBoxes(const wxString &odir, const v4d::MapOfPositionBoxes &mp) = 0;
		/** Has cropper any parameter to set? If so, function returns true. */
		virtual bool HasParams() = 0;
		/** Set cropping parameters. */
		virtual void SetParams(wxWindow *parent) = 0;
		/** Are all of cropper parameters set? If this function returns true cropping can be started. */
		virtual bool CroppingAvail() = 0;
		/** Crop given boxes from given position. */
		virtual void CropPosition(const wxString &position, const v4d::ListOfBoxes &boxes) = 0;
	};
}

#endif
