/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * bcrop_image_cropper.h
 *
 * This file defines ImageCropperBySlices class
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006
 */

#ifndef __IMAGE_CROPPER_BY_SLICES_H__
#define __IMAGE_CROPPER_BY_SLICES_H__

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <i3d/resolution.h>
#include "image_cropper.h"

#include "dlg_channels_bcrop.h"

namespace bcrop {	

	/** The ImageCropperBySlices class represents the image cropper for batchcrop. */ 
	class ImageCropperBySlices : public ImageCropper<i3d::RGB>
	{
	  private:
        /** Output directory. */
        wxString m_odir;
		/** Must cropper delete old data during cropping the box that has already been cropped? */
		bool m_delete_old;
		/** Image resolution. */
		i3d::Resolution m_res;
		/** List of defined channels. */
		ChannelList m_channels;

	  public:
		/** Constructor. */
		ImageCropperBySlices(): ImageCropper<i3d::RGB>(), m_odir(wxEmptyString), m_delete_old(true), m_res(), m_channels() {};
		/** Destructor. */
		virtual ~ImageCropperBySlices() {}; 
		/** Load boxes for all positions from output directory. Function returns true if loading was successful. */
		virtual bool LoadAllBoxes(const wxString &odir, v4d::MapOfPositionBoxes &mp);
		/** Save boxes for all positions to output directory. Function returns true if saving was successful. */
		virtual bool SaveAllBoxes(const wxString &odir, const v4d::MapOfPositionBoxes &mp);
		/** Has cropper any parameter to set? If so, function returns true. */
		virtual bool HasParams();
		/** Set cropping parameters. */
		virtual void SetParams(wxWindow *parent);
		/** Are all of cropper parameters set? If this function returns true cropping can be started. 
		This function returns true if cropper has any channel with defined colour component, i.e. colour 
		component of channel isn't cc_NONE. */
		virtual bool CroppingAvail();		
		/** Crop given boxes from given position. */
		virtual void CropPosition(const wxString &position, const v4d::ListOfBoxes &boxes);

	  private:
	    /** Return true if any channel has the only one colour component, i.e. colour 
		component of channel is neither cc_NONE nor cc_RGB. */
        bool IsDecompositionNeeded();
        /** Crop boxes from given slice (2D image). */
		void CropSingleSlice(const wxString &fname, const v4d::ListOfBoxes &boxes, bool use_decomposition);
		/** Prepare box for cropping, i.e. ensure its positive size and positive offset. */
		void PrepareBox(v4d::Box<float> &box);

		//--------------------------------------------------------------------------
		// Channels related functions
		//--------------------------------------------------------------------------
		/** Load the names of channels from the output directory (from the file channels.cfg). */
		void LoadChannels();
		/** Save channels to the output directory (to the file channels.cfg). */
		void SaveChannels();
		/** Show channel dialog	and save the modified channels to the output 
		directory (to the file channels.cfg). */
		void ShowChannelDialog(wxWindow *parent);
	};

} // namespace bcrop

#endif
