/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
* overlaying.h
*
* Overlaying functions
*
* Martin Maka (xmaska@fi.muni.cz) 2007
*/

#ifndef V4D_OVERLAYING_H
#define V4D_OVERLAYING_H

#include <i3d/hrca_lut.h>
#include "v4d_impexp.h"
#include "types/extension.h"
#include "types/overlayer.h"

namespace v4d 
{
    //////////////////////////////////////////////////////////////////////////////////////////
    //
    // v4d_OverlayingModule
    // This class class provides all overlaying functions needed by the af_frame and v4d_frame.
    //
    //////////////////////////////////////////////////////////////////////////////////////////
	template <class LABEL, class VOXEL> 
		class V4D_DLLEXPORT v4d_OverlayingModule : public v4d_ExtensionModule, public v4d_Overlayer<LABEL, VOXEL>
	{
  	  protected:
		/** Is a colour mode default (default palette) or user-defined (one selected colour)? */
		bool m_default_colour_mode;
		/** An user-defined colour. */
		wxColour m_user_colour;
        /** Pointer to overlaying menu */
        wxMenu *m_menu_ptr;

 	  public:
		/** Constructor. */
		v4d_OverlayingModule(wxFrame *parent);
        /** Destructor */
		virtual ~v4d_OverlayingModule() 
        {}

        /** Create a module menu. */
		virtual wxMenu* CreateMenu(int &last_id);

      protected:
        /** Return the color for given label */
        virtual i3d::RGB GetLabelColour (LABEL label, size_t buf_id) const 
        { 
            return this->m_pal[label % this->m_pal_size]; 
        }

	  private:
        /** Handler for module menu events. */
		void OnOverlaying(wxCommandEvent &evt); 
		/** Handler for user interface events. */
		void OnUpdateUI(wxUpdateUIEvent &evt);
		/** Change an user-defined colour. This function also changes the LUT table 
		according to the selected colour mode. */
		void ChangeColour();
		/** Set an user-defined colour. */
		void SetColour(const wxColour &colour) { m_user_colour = colour; }
        /** Set LUT according to the selected colour mode. */
		void SetPal();
	};
}

#endif


