/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * box.h
 *
 * templates for rectangular boxes with ids
 *
 */

#ifndef V4D_TYPES_BOX_H
#define V4D_TYPES_BOX_H

#include <list>
#include <map>
#include "roi.h"

namespace v4d
{
    /** Box is a rectangle with assigned id */
    template <class T> struct Box
        : public ROI<T>
    {
    public:
        /** Identification number of the box */
        unsigned int id_box;

        using ROI<T>::offset;
        using ROI<T>::size;

    public:
        /** Constructor */
        Box()
            : ROI<T>(), id_box(0) 
        {}

        /** Constructor */
        Box(int id, T x_ = 0, T y_ = 0, T dx_ = 0, T dy_ = 0)
            : ROI<T>(x_, y_, dx_, dy_), id_box(id) 
        {}


        void PrepareForResizing(const Vector2d<T> &pos)
        {
            EnsurePositiveSize(*this);
            if (std::abs(static_cast<int>(pos.x - offset.x)) < std::abs(static_cast<int>(pos.x - offset.x - size.x)))
            {
                offset.x = offset.x + size.x;
                size.x = -size.x;
            }

            if (std::abs(static_cast<int>(pos.y - offset.y)) < std::abs(static_cast<int>(pos.y - offset.y - size.y)))
            {
                offset.y = offset.y + size.y;
                size.y = -size.y;
            }
        }
    };

    /** List of crop boxes. Each image in the array of input 
        images has one CropAreaList
    */
    typedef std::list< Box<float> > ListOfBoxes;
    typedef std::map<wxString, ListOfBoxes> PositionBoxes;
    typedef std::map<wxString, PositionBoxes> MapOfPositionBoxes;
}

#endif
