/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * cutpointer.h
 *
 * Cut pointer types
 *
 * Petr Matula (pem@fi.muni.cz) 2006
 */


#ifndef V4D_TYPES_CUTPOINTER_H
#define V4D_TYPES_CUTPOINTER_H

#include <wx/colour.h>

#define CP_SHORT_LEN    10

namespace v4d 
{
	/** Cut pointer types */
	enum v4d_CutPointerType
	{
		cp_NONE,   ///< Hide pointer 
		cp_SHORT,  ///< Show pointer as short dashes on borders
		cp_CIRCLE, ///< Show pointer as short dashes on borders and circle at cut position
		cp_DOTTED, ///< Show pointer as a dotted cross
		cp_DASHED, ///< Show pointer as a dashed cross
		cp_FULL    ///< Show pointer as a solid line cross
	};

	/** Cut pointer style */
	struct v4d_CutPointerStyle
	{
		/** Type of the pointer */
		v4d_CutPointerType type;
		/** Colour of the pointer */
		wxColour colour;

		/** Default constructor */
		v4d_CutPointerStyle(v4d_CutPointerType t = cp_SHORT, const wxColour &col = wxColour(0, 255, 0))
            : type(t), colour(col)
        {}
	};
}

#endif
