/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * datareader.h
 *
 * Abstract class for reading data in afframe
 *
 * Petr Matula (pem@fi.muni.cz) 2006
 */

#ifndef V4D_TYPES_DATAREADER_H
#define V4D_TYPES_DATAREADER_H

#include <i3d/image3d.h>
#include <i3d/regions.h>
#include "overlayer.h"
#include "../thistogram.h"

namespace afframe 
{
    /** Abstract class for reading data required by AFFrame */
    template <class T>
    class af_DataReader 
    {
    public:
        /** Default constructor */
        af_DataReader() 
        {}
        
        /** Default destructor */
        virtual ~af_DataReader() 
        {}

        /** Redefine this function to load af image data corresponding
            to position pos to the variable rgb. It should return
            true on success. */
        virtual bool LoadAFImage(i3d::Image3d<T> &rgb, const wxString &pos) = 0;

        /** Redefine this function to load 3D image data corresponding
            to position pos and rectangle rect to the variable rgb. It should return
            true on success. */
        virtual bool Load3dImage(i3d::Image3d<T> &img, const wxString &pos, const wxRect &rect) = 0;

        /** Redefine this function to load image histogram (for example from a hard disc). This
            method can be used to speed up the visualization process. Return \c false if the
            histogram was not loaded or on error. */
        virtual bool LoadHistogram(const wxString &pos, v4d::HistogramSet<T> &hist_set) 
        { 
            return false; 
        }
    };
}

#endif

