/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * evt_mapping.h
 *
 * Event signaling changes in mapping
 *
 * Petr Matula (pem@fi.muni.cz) 2006
 */


#ifndef V4D_TYPES_EVT_MAPPING_H
#define V4D_TYPES_EVT_MAPPING_H

#include <wx/event.h>
#include "../v4d_impexp.h"

namespace v4d 
{
    /**************************************************/
    /*                                                */
    /*               Mapping events                   */
    /*                                                */
    /**************************************************/
            
    /** An event sent by LevelsWidget dialog as a request for preview drawing. */
    class v4d_MappingEvent 
        : public wxEvent
    {
    public:
        bool _refreshAll;

    public:
        /** Constructor. */
        v4d_MappingEvent(wxEventType eventType, int id = 0, bool refreshAll = true)
            : wxEvent(id, eventType), _refreshAll(refreshAll) 
        {}

        /** Clone method. */
        virtual wxEvent *Clone() const
        { 
            return new v4d_MappingEvent(*this);
        }        
    };
    
    /** An event sent by v4d_MappingModule to inform parent about taken changes. */
    class v4d_MappingCommandEvent 
        : public wxCommandEvent
    {
    private:
        wxEvent *_attEvent;

    public:
        /** Constructor. */
        v4d_MappingCommandEvent(wxEventType eventType, int id = 0)
            : wxCommandEvent(eventType, id), _attEvent(NULL) 
        {}

        /** Copy constructor */
        v4d_MappingCommandEvent(const v4d_MappingCommandEvent &copy)
            : wxCommandEvent(copy)
        { 
            _attEvent = copy._attEvent->Clone(); 
        }
        
        /** Destructor */
        ~v4d_MappingCommandEvent() 
        { 
            if (_attEvent != NULL) 
            {
                delete _attEvent; 
            }
        }

        /** Clone method. */
        virtual wxEvent *Clone() const 
        { 
            return new v4d_MappingCommandEvent(*this); 
        }

        /** Get the attached event */
        wxEvent* GetAttachedEvent() 
        { 
            return _attEvent; 
        }

        /** Get the attached event */
        void SetAttachedEvent(const wxEvent &evt) 
        { 
            if (_attEvent != NULL)
            {
                delete _attEvent; 
            }

			_attEvent = evt.Clone(); 
        }
    };

    // Declare event tags
    wxDECLARE_EXPORTED_EVENT(V4D_DLLEXPORT, v4d_EVT_MAPPING_CHANGED, v4d_MappingEvent);
    wxDECLARE_EXPORTED_EVENT(V4D_DLLEXPORT, v4d_EVT_MAPPING_COMMAND, v4d_MappingCommandEvent);
}

#endif

