/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * prior_log.h
 *
 * Priority logger
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006
 */

#ifndef V4D_PRIOR_LOG_H
#define V4D_PRIOR_LOG_H

#include <wx/log.h> 

namespace v4d
{
    /** Class providing a logger whose priority can be changed independently of priorities of others loggers. */
    class PriorityLog: public wxLogGui
    {
    private:
        /** The log priority. */
        wxLogLevel m_level;

    public:
        /** Constructor. */
        PriorityLog(wxLogLevel level): wxLogGui(), m_level(level) {};

    protected:
        virtual void DoLogRecord(wxLogLevel level, const wxString &msg, const wxLogRecordInfo &info)
        {
            if (level <= m_level)
                wxLogGui::DoLogRecord(level, msg, info);

            if (level == wxLOG_Debug)
            {
                wxLogMessage("Debug: %s", msg.c_str());
            }
        }
    };
}

#endif
