/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * projections.h
 *
 * Projections in viewer
 *
 * Petr Matula (pem@fi.muni.cz) 2006
 */

#ifndef V4D_TYPES_PROJECTIONS_H
#define V4D_TYPES_PROJECTIONS_H

namespace v4d 
{
    /////////////////////////////////////////////////////////////////
    //                                                             //
    //                       Projection Type                       //
    //                                                             //
    /////////////////////////////////////////////////////////////////
    /** Enumeration type specifying the required projection. It is used in
        functions that work with projections: e.g. displaying, computation of
        afs, etc. The identifiers are chosen so that constructions like if
        (pt & pt_XY), can be used. Where pt is a variable of type
        ProjectionType. */
    enum v4d_ProjectionType
    {
        pt_None = 0,     /**< No projection */
        pt_XY = 0x01,    /**< XY projection */
        pt_XZ = 0x02,    /**< XZ projection */
        pt_YZ = 0x04,    /**< YZ projection */
        pt_ButXY = 0x06, /**< All projections but XY */
        pt_ButXZ = 0x05, /**< All projections but XZ */
        pt_ButYZ = 0x03, /**< All projections but YZ */
        pt_All = 0x07    /**< All projections */
    };

    ///////////////////////////////////////////////////////////
    //                                                       //
    //                  Projection content                   //
    //                                                       //
    ///////////////////////////////////////////////////////////
    /** Types of the content in the projection windows. */
    enum v4d_ProjectionContent
    {
        /** The window displays a classical orthogonal projection. */
        pc_Ortho,
        /** The window displays an autofocus projection, i.e. the maximum of the
            intensities along the axis perpendicular to the plane of the
            window. */
        pc_Af
    };
}

#endif
