/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * thresparams.h
 *
 * Parameters of thresholding dialog
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006
 */


#ifndef V4D_TYPES_THRESPARAMS_H
#define V4D_TYPES_THRESPARAMS_H

#include <limits>

namespace v4d
{
    template <class T> 
    struct ThresParams
    {
        T min_threshold;
        T max_threshold;

        /** Constructor. */
        ThresParams(T min = std::numeric_limits<T>::min(), T max = std::numeric_limits<T>::max())
            : min_threshold(min), max_threshold(max) 
        {}

        ThresParams(const ThresParams<float> &params)
        {
            min_threshold = static_cast<T>(params.min_threshold);
            max_threshold = static_cast<T>(params.max_threshold);
        }

        /** Convert values to float. */
        ThresParams<float> ConvertToFloat() const
        {
            return ThresParams<float>(static_cast<float>(min_threshold), static_cast<float>(max_threshold));
        }
    };
}

#endif
