/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * tlevelsparams.h
 *
 * Parameters of levels dialog
 *
 * Petr Matula (pem@fi.muni.cz) 2006
 * Martin Maka (xmaska@fi.muni.cz) 2008
 */


#ifndef V4D_TYPES_TLEVELSPARAMS_H
#define V4D_TYPES_TLEVELSPARAMS_H

#include <limits>

namespace v4d
{
    template <class ITYPE, class OTYPE> 
    struct TLevelsParams
    {
        /** A minimal input value. */
        ITYPE inputMin;
        /** A maximal input value. */
        ITYPE inputMax;
        /** A magnitude of the gamma correction. */
        double inputGamma;
        /** A minimal output value. */
        OTYPE outputMin;
        /** A maximal output value. */
        OTYPE outputMax;

        /** Constructor. */
        TLevelsParams(
            ITYPE imin = std::numeric_limits<ITYPE>::min(), 
            ITYPE imax = std::numeric_limits<ITYPE>::max(), 
            double gamma = 1.0, 
            OTYPE omin = std::numeric_limits<OTYPE>::min(), 
            OTYPE omax = std::numeric_limits<OTYPE>::max()) 
            : inputMin(imin), inputMax(imax), inputGamma(gamma), outputMin(omin), outputMax(omax) 
        {}

        TLevelsParams(const TLevelsParams<float, float> &params)
        {
            inputMin = static_cast<ITYPE>(params.inputMin);
            inputMax = static_cast<ITYPE>(params.inputMax);
            inputGamma = params.inputGamma;
            outputMin = static_cast<OTYPE>(params.outputMin);
            outputMax = static_cast<OTYPE>(params.outputMax);
        }

        /** Convert the input and output values to float. */
        TLevelsParams<float, float> ConvertToFloat() const
        {
            return TLevelsParams<float, float>(
                static_cast<float>(inputMin),
                static_cast<float>(inputMax),
                inputGamma,
                static_cast<float>(outputMin),
                static_cast<float>(outputMax));
        }
    };
}

#endif
