/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * viewervars.h
 *
 * Variables needed in 3d viewer
 *
 * Petr Matula (pem@fi.muni.cz) 2003
 */


#ifndef V4D_VIEWERVARS_H
#define V4D_VIEWERVARS_H

#include <i3d/voi.h>
#include <i3d/vector3d.h>
#include "projections.h"

namespace v4d 
{
    /////////////////////////////////////////////////////////////////
    //                                                             //
    //                         Viewer Vars                         //
    //                                                             //
    /////////////////////////////////////////////////////////////////
    /** This class puts together (for convenience) several variables that
    are used in all three projection windows.  */
    struct v4d_ViewerVars {
        /** Bounding box of the loaded image in micrometers. */
        i3d::VOI<i3d::MICRONS> bbox;

        /** Visible part of the image in micrometers. This variable is
            changed when size of a drawing window is changed. \sa
            Viewer3dCanvas::RecomputeVisible(). */
        i3d::VOI<i3d::MICRONS> vis3d;

        /** Current resolution of the screen in pixels per micrometer. The
            resolution is typically changed on zooming. */
        i3d::Vector3d<float> res;

        /** Original resolution of the screen that is set when a new image
            is loaded. This resolution is copied to the current resolution
            whenever the zoom 1:1 is requested. */
        i3d::Vector3d<float> orig_res;

        /** Current zoom. zoom_level == 0 => 1:1; 
            zoom_level > 0  => 1:n+1; 
            zoom_level < 0  => n+1:1
            The current zoom ratio could be computed from res and orig_res 
            variables, however, int variable is rather used to prevent from rounding
            errors */
        int zoom_level;

        /** Position of the cut in micrometers. It corresponds to an
            intersection point of all three orthogonal planes. */
        i3d::Vector3d<float> cut;

        /** This variable says how many microns are added to (or subtracted
            from) vis3d when a cursor key is pressed in a drawing window. */
        i3d::Vector3d<float> mstep;

        /** This variable says how many microns are added to (or subtracted
            from) cut when a PgUp or PgDown key is pressed in a drawing
            window. It is recomended to set the z commponent to 
            the distance between slices.  */
        i3d::Vector3d<float> cstep;

        /** Which axis the user wants to be reversed? 0 - XY, 1 - XZ, 2 - YZ*/
        bool rev_axis[3];

		  /** The actual time point selected in the sequence. */
		  size_t time;

        /** Constructor initialize the structure. */
        v4d_ViewerVars(): bbox(0, 0, 0, 20, 20, 7), res(10, 10, 10), orig_res(res), 
            zoom_level(0), mstep(1,1,1), cstep(0.1f,0.1f,0.2f), time(0)
        { 
            rev_axis[0] = rev_axis[1] = rev_axis[2] = false;
        }
    };
}

#endif
