/*
*  v4dwins
*
*  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Lesser General Public
*  License as published by the Free Software Foundation; either
*  version 2 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with this library; if not, write to the Free
*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*-*-c++-*-
* v4d_viewer3d.h
*
* 3d image viewer frame
*
* Petr Matula (pem@fi.muni.cz) 2006
*/

#include "v4d_basic.h"
#include "v4d_frame.h"
#include "v4d_imagecanvas3d.h"
#include "v4d_labeledimagecanvas3d.h"
#include "mapping.h"
#include "dlg_pref.h"
#include "dlg_decor.h"
#include "dlg_gallery.h"
#include "wdr/dlg_decor_wdr.h"
#include "pref.h"
#include "basic_overlayer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

namespace v4d
{
	wxColourData v4d_Preferences::m_colour_palette;

	/////////////////////////////////////////////////////////////////
	//                                                             //
	//                          Event tables                       //
	//                                                             //
	/////////////////////////////////////////////////////////////////
	// the event tables connect the wxWindows events with the functions (event
	// handlers) which process them. It can be also done at run-time, but for the
	// simple menu events like this the static method is much simpler.
	BEGIN_EVENT_TABLE(v4d_Viewer3dFrame, wxFrame)
		EVT_MENU(SI_SM_PLAY_FORWARD, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_PLAY_BACKWARD, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_PLAY_STOPNOW, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_SETRANGE, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_FPS_INCREASE, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_FPS_DECREASE, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_STEP_INCREASE, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_STEP_DECREASE, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_PLAY_LOOP, v4d_Viewer3dFrame::OnSequenceMenuCmd)
		EVT_MENU(SI_SM_PLAY_OSCILLATION, v4d_Viewer3dFrame::OnSequenceMenuCmd)

		EVT_MENU(SI_TOGGLE_XY, v4d_Viewer3dFrame::OnToggleProjection)
		EVT_MENU(SI_TOGGLE_XZ, v4d_Viewer3dFrame::OnToggleProjection)
		EVT_MENU(SI_TOGGLE_YZ, v4d_Viewer3dFrame::OnToggleProjection)
		EVT_MENU(SI_AF, v4d_Viewer3dFrame::OnAutoFocus)
		EVT_MENU(SI_SAVE_3D_DATA, v4d_Viewer3dFrame::OnSave3dData)
		EVT_MENU(SI_SAVE_VIEW, v4d_Viewer3dFrame::OnSaveView)
		EVT_MENU(SI_SAVE_GALLERY, v4d_Viewer3dFrame::OnSaveGalleryView)
		EVT_MENU(SI_ADJUST_FRAME_SIZE, v4d_Viewer3dFrame::OnAdjustFrameSize)
		EVT_MENU(SI_VISUALIZE_VOLUME, v4d_Viewer3dFrame::OnVisualizeVolume)
		EVT_MENU(SI_ZOOM_IN, v4d_Viewer3dFrame::OnZoomIn)
		EVT_MENU(SI_ZOOM_OUT, v4d_Viewer3dFrame::OnZoomOut)
		EVT_MENU(SI_NORMAL_ZOOM, v4d_Viewer3dFrame::OnNormalZoom)
		EVT_MENU(SI_CUT, v4d_Viewer3dFrame::OnPreferences)
		EVT_MENU(SI_REVERSE_X, v4d_Viewer3dFrame::OnReverseAxis)
		EVT_MENU(SI_REVERSE_Y, v4d_Viewer3dFrame::OnReverseAxis)
		EVT_MENU(SI_REVERSE_Z, v4d_Viewer3dFrame::OnReverseAxis)
		EVT_MENU(SI_AUTOFOCUS_XY, v4d_Viewer3dFrame::OnAutoFocus)
		EVT_MENU(SI_AUTOFOCUS_XZ, v4d_Viewer3dFrame::OnAutoFocus)
		EVT_MENU(SI_AUTOFOCUS_YZ, v4d_Viewer3dFrame::OnAutoFocus)
		EVT_MENU(SI_SHOW_WINDOWS_ANNOTATIONS, v4d_Viewer3dFrame::OnShowWindowsAnnotation)
		EVT_MENU(SI_SHOW_INFO_WIN, v4d_Viewer3dFrame::OnShowInfo)
		EVT_UPDATE_UI_RANGE(SI_FIRST_UPDATE_UI, SI_LAST_UPDATE_UI, v4d_Viewer3dFrame::OnUpdateUI)
		END_EVENT_TABLE()

		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                       Viewer3d Frame                        //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		v4d_Viewer3dFrame::v4d_Viewer3dFrame(wxWindow* parent, wxWindowID id,
		const wxString& title, const wxPoint& pos, const wxSize& size,
		long style)
		: wxFrame(parent, id, title, pos, size, style),
		m_last_menu_id(SI_LAST_MENU_ID), m_canvas(NULL), m_status_cleaner(this)
	{
		////////////////////////////////
		// Menu Sequence
		////////////////////////////////
		wxMenu *menuSeq = new wxMenu;
		menuSeq->Append(SI_SM_PLAY_FORWARD, "Browse forward\tCtrl+F", "Starts to browse the sequence automatically as a slideshow.",wxITEM_CHECK);
		menuSeq->Append(SI_SM_PLAY_BACKWARD, "Browse backward\tCtrl+B", "Starts to browse the sequence automatically as a slideshow.",wxITEM_CHECK);
		menuSeq->Append(SI_SM_PLAY_STOPNOW, "Stop browsing\tCtrl+T", "Stops the automatic browsing immediately.");
		menuSeq->AppendSeparator();
		menuSeq->Append(SI_SM_PLAY_LOOP, "Loop mode", "When end of the sequence is reached, should automatic browsing start again?", wxITEM_CHECK);
		menuSeq->Append(SI_SM_PLAY_OSCILLATION, "Oscillation mode", "When end of the sequence is reached, should automatic browsing continue from the beginning or in an opposite direction?", wxITEM_CHECK);
		menuSeq->AppendSeparator();
		menuSeq->Append(SI_SM_SETRANGE, "Set browsing range...", "Open dialog for setting the image ranges");
		menuSeq->Append(SI_SM_FPS_INCREASE, "Increase browsing speed\tF", "Increases FPS (images per second) automatic browsing speed.");
		menuSeq->Append(SI_SM_FPS_DECREASE, "Decrease browsing speed\tS", "Decreases FPS (images per second) automatic browsing speed.");
		menuSeq->Append(SI_SM_STEP_INCREASE, "Increase browsing step", "Increases number of images that are stepped over when moving to a next image.");
		menuSeq->Append(SI_SM_STEP_DECREASE, "Decrease browsing step", "Decreases number of images that are stepped over when moving to a next image.");

		////////////////////////////////
		// Menu View
		////////////////////////////////
		wxMenu *menuView = new wxMenu;
		menuView->Append(SI_SAVE_3D_DATA, "Save 3D data...\tCtrl+S", "Save original 3D image data");
		menuView->Append(SI_SAVE_VIEW, "Save view...\tCtrl+V", "Save current view for publication purposes");
		menuView->Append(SI_SAVE_GALLERY, "Save gallery...\tCtrl+I", "Save slices of 3D data in chosen increment");
		menuView->AppendSeparator();
		menuView->Append(SI_ADJUST_FRAME_SIZE, "Adjust frame size\tCtrl+E", "Adjust frame size according to the visible image");
		menuView->Append(SI_AUTO_ADJUST_FRAME_SIZE, "Auto adjust frame size", "Automatically adjust frame on zooming", wxITEM_CHECK);
#ifndef VLADO_PRIVATE_TWEAKS
		menuView->Check(SI_AUTO_ADJUST_FRAME_SIZE, true);
#endif
		menuView->AppendSeparator();
		menuView->Append(SI_VISUALIZE_VOLUME, "3D volume visualization\tV", "Show 3D visualization");
		menuView->AppendSeparator();
		menuView->Append(SI_ZOOM_IN, "Zoom In\tF2", "Zoom in");
		menuView->Append(SI_ZOOM_OUT, "Zoom Out\tF3", "Zoom out");
		menuView->Append(SI_NORMAL_ZOOM, "Normal Zoom\t1", "Normal zoom");
		menuView->AppendSeparator();
		menuView->Append(SI_AF, "Autofocus in all windows\tA", "Autofocus in all windows on/off", true);
		menuView->AppendSeparator();
		menuView->Append(SI_SHOW_WINDOWS_ANNOTATIONS, "Show windows annotation\tAlt+/", "Show/hide windows annotation", wxITEM_CHECK);
		menuView->Append(SI_SHOW_INFO_WIN, "Show info\tI", "Show/hide info window", wxITEM_CHECK);
		menuView->AppendSeparator();

		wxMenu *menuPanel = new wxMenu;
		menuPanel->Append(SI_TOGGLE_XY, "Show XY\tZ", "Show/hide XY window", wxITEM_CHECK);
		menuPanel->Append(SI_TOGGLE_XZ, "Show XZ\tY", "Show/hide XZ window", wxITEM_CHECK);
		menuPanel->Append(SI_TOGGLE_YZ, "Show YZ\tX", "Show/hide YZ window", wxITEM_CHECK);
		menuPanel->Check(SI_TOGGLE_XY, true);
		menuPanel->Check(SI_TOGGLE_XZ, true);
		menuPanel->Check(SI_TOGGLE_YZ, true);
		menuView->Append(SI_WINDOW, "Window", menuPanel, "Show/hide window");

		wxMenu *menuAxis = new wxMenu;
		menuAxis->Append(SI_REVERSE_X, "Reverse X\tF9", "Reverse X axis", wxITEM_CHECK);
		menuAxis->Append(SI_REVERSE_Y, "Reverse Y\tF10", "Reverse Y axis", wxITEM_CHECK);
		menuAxis->Append(SI_REVERSE_Z, "Reverse Z\tF11", "Reverse Z axis", wxITEM_CHECK);
		menuView->Append(SI_AXIS, "Axis", menuAxis, "Axis options");

		wxMenu* menuAutofocus = new wxMenu;
		menuAutofocus->Append(SI_AUTOFOCUS_XY, "Window XY", "Autofocus in XY windows on/off", wxITEM_CHECK);
		menuAutofocus->Append(SI_AUTOFOCUS_XZ, "Window XZ", "Autofocus in XZ windows on/off", wxITEM_CHECK);
		menuAutofocus->Append(SI_AUTOFOCUS_YZ, "Window YZ", "Autofocus in YZ windows on/off", wxITEM_CHECK);
		menuAutofocus->Check(SI_AUTOFOCUS_XY, false);
		menuAutofocus->Check(SI_AUTOFOCUS_XZ, false);
		menuAutofocus->Check(SI_AUTOFOCUS_YZ, false);
		menuView->Append(SI_AUTOFOCUS_XYZ, "Autofocus", menuAutofocus, "Switch autofocus on/off");

		menuView->AppendSeparator();
		menuView->Append(SI_CUT, "Preferences\tCtrl-P", "Change screen options");

		////////////////////////////////
		// MenuBar
		////////////////////////////////
		wxMenuBar *menuBar = new wxMenuBar;
		menuBar->Append(menuSeq, "&Sequence");
		menuBar->Append(menuView, "&View");

		// ... and attach this menu bar to the frame
		SetMenuBar(menuBar);

		// create status bar with 1 pane
		CreateStatusBar(2);
		int StatusBarWidths[2]={-1,180};
		GetStatusBar()->SetStatusWidths(2,StatusBarWidths);

		////////////////////////////////
		// More keyboard shortcuts
		////////////////////////////////
		wxAcceleratorEntry ac_ent[20]; //slightly inefficient
		int ac_entrs=0;

		//standard accelerators
		ac_ent[ac_entrs++].Set(wxACCEL_CTRL, WXK_NUMPAD_ADD, SI_ZOOM_IN);
		ac_ent[ac_entrs++].Set(wxACCEL_CTRL, WXK_NUMPAD_SUBTRACT, SI_ZOOM_OUT);

		#if defined(__UNIX__) && defined(__WXGTK__)
			//we're on Linux/Unix with GTK-based wxWidgets implementation:
			//as of wxWidgets-3.0.2, this particular configuration was observed
			//to ignore single-char menu accelerators (such as 'a', 'v', 'i',...),
			//here, we just need to treat these explicitly:

			//"patching" accelerators
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'V', SI_VISUALIZE_VOLUME);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'1', SI_NORMAL_ZOOM);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'A', SI_AF);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'I', SI_SHOW_INFO_WIN);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'Z', SI_TOGGLE_XY);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'Y', SI_TOGGLE_XZ);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'X', SI_TOGGLE_YZ);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'F', SI_SM_FPS_INCREASE);
			ac_ent[ac_entrs++].Set(wxACCEL_NORMAL, (int)'S', SI_SM_FPS_DECREASE);

			wxLogMessage(wxString("Detected wxGTK on Linux: registering some more accelerator keys."));
		#endif

		wxAcceleratorTable ac_tab(ac_entrs, ac_ent);
		SetAcceleratorTable(ac_tab);

		////////////////////////////////
		// Bind events
		////////////////////////////////
		Bind(wxEVT_MOVE, &v4d_Viewer3dFrame::OnMove, this);
		Bind(v4d_EVT_MAPPING_CHANGED, &v4d_Viewer3dFrame::OnMappingChanged, this);
		Bind(v4d_EVT_OVERLAYING_CHANGED, &v4d_Viewer3dFrame::OnOverlayingChanged, this);

		SetSizeHints(200, 200);
		ResizeFrameIfRequired();
	}

	v4d_Viewer3dFrame::~v4d_Viewer3dFrame()
	{
		// delete all extension modules
		//wxEvtHandler *eh;
		while (!m_delete_extension.empty()){
			PopEventHandler(m_delete_extension.top());
			m_delete_extension.pop();
		};

		wxUpdateUIEvent e(SI_CLOSE_WINDOW);
		e.SetClientData(this);
		GetParent()->ProcessWindowEvent(e);
	}

	void v4d_Viewer3dFrame::SetCanvas(v4d_Viewer3dCanvas *canvas)
	{
		if (m_canvas)
		{
			m_canvas->Destroy();
		}

		m_canvas = canvas;
		m_canvas->SetSize(GetClientSize());
	}

	void v4d_Viewer3dFrame::OnToggleProjection(wxCommandEvent& event)
	{
		v4d_ProjectionType pt;

		switch (event.GetId())
		{
		case SI_TOGGLE_XY: pt = pt_XY; break;
		case SI_TOGGLE_XZ: pt = pt_XZ; break;
		case SI_TOGGLE_YZ: pt = pt_YZ; break;
		default: return;
		}

		m_canvas->ShowDrWin(pt, !m_canvas->IsDrWinShown(pt));
	}

	void v4d_Viewer3dFrame::OnShowWindowsAnnotation(wxCommandEvent& evt)
	{
		m_canvas->ToggleInfoFlag();
		m_canvas->Refresh(false);
	}

	void v4d_Viewer3dFrame::OnShowInfo(wxCommandEvent& evt)
	{
		m_canvas->InfoFrame_Show(!m_canvas->InfoFrame_IsShown());
	}

	void v4d_Viewer3dFrame::RefreshIcon()
	{
		wxIconBundle bundle;
		m_canvas->GetIcons(bundle);
		SetIcons(bundle);
		m_canvas->RefreshVolVisIcon(bundle);
	}

	void v4d_Viewer3dFrame::OnAutoFocus(wxCommandEvent& event)
	{
		//what AF direction are we talking about?
		v4d_ProjectionType what = pt_All;
		switch (event.GetId())
		{
		case SI_AF:
			what = pt_All;
			break;
		case SI_AUTOFOCUS_XY:
			what = pt_XY;
			break;
		case SI_AUTOFOCUS_XZ:
			what = pt_XZ;
			break;
		case SI_AUTOFOCUS_YZ:
			what = pt_YZ;
			break;
		default:
			what = pt_All;
			break;
		}

		//toggle its state...
		v4d_ProjectionContent state =
			(m_canvas->GetContent(what) == pc_Af)? pc_Ortho : pc_Af;
		m_canvas->ChangeContent(state, what);
	}

	void v4d_Viewer3dFrame::OnMappingChanged(v4d_MappingEvent& evt)
	{
		m_canvas->MappingChanged(evt);
	}

	void v4d_Viewer3dFrame::OnOverlayingChanged(v4d_OverlayingEvent& evt)
	{
		m_canvas->OverlayingChanged(evt);
	}

	void v4d_Viewer3dFrame::OnSave3dData(wxCommandEvent &event)
	{
		//creates the dialog window and populates it...
		wxDialog dlg(this, -1, "Save 3D image data as...", wxDefaultPosition, wxDefaultSize);
		SaveDataDialogFunc(&dlg);

		//attaches a helper object that can handle 'Sequence params' dialog items
		ExportSequenceDialog s_dlg(&dlg,m_canvas);

		if (dlg.ShowModal() == wxID_OK)
		{
			ExportSave3dData_CanvasProcessor workhorse(m_canvas);
			s_dlg.ProcessAfterShowModal(workhorse);
		}
	}

	void v4d_Viewer3dFrame::OnSaveView(wxCommandEvent &event)
	{
		//creates the dialog window and populates it...
		long int d = 10;
		DecorationDialog d_dlg(this, d);

		//attaches a helper object that can handle 'Sequence params' dialog items
		ExportSequenceDialog s_dlg(&d_dlg,m_canvas);

		if (d_dlg.ShowModal() == wxID_OK)
		{
			d_dlg.TransferDataFromWindow();
			v4d_ProjectionType proj = d_dlg.GetSelectedProjection();

			ExportSaveActualView_CanvasProcessor workhorse(m_canvas,static_cast<int>(d), proj);
			s_dlg.ProcessAfterShowModal(workhorse);
		}
	}

	void v4d_Viewer3dFrame::OnSaveGalleryView(wxCommandEvent &event)
	{
		//creates the dialog window and populates it...
		GalleryDialog dlg(this, int(m_canvas->GetImageSize().z));

		//attaches a helper object that can handle 'Sequence params' dialog items
		ExportSequenceDialog s_dlg(&dlg,m_canvas);

		if (dlg.ShowModal() == wxID_OK)
		{
			ExportSaveGalleryView_CanvasProcessor workhorse(m_canvas,
				dlg.GetStartOffset(), dlg.GetEndOffset(),
				dlg.GetStep(), dlg.IsFixedRows(), dlg.GetCount(), dlg.GetSpacing());
			s_dlg.ProcessAfterShowModal(workhorse);
		}
	}

	void v4d_Viewer3dFrame::OnAdjustFrameSize(wxCommandEvent& evt)
	{
		if (!m_canvas)
			return;

		wxRect rect;
		rect = wxGetClientDisplayRect();

		wxSize minsz(100, 100);
		wxSize maxsz(rect.width, rect.height);

		wxSize sz = m_canvas->GetOptimalSize();
		sz = ClientToWindowSize(sz);

		if (sz.GetWidth() < minsz.GetWidth())
			sz.SetWidth(minsz.GetWidth());
		if (sz.GetHeight() < minsz.GetHeight())
			sz.SetHeight(minsz.GetHeight());

		if (sz.GetWidth() > maxsz.GetWidth())
			sz.SetWidth(maxsz.GetWidth());
		if (sz.GetHeight() > maxsz.GetHeight())
			sz.SetHeight(maxsz.GetHeight());

		if (!IsMaximized())
			SetSize(sz);
	}

	void v4d_Viewer3dFrame::OnVisualizeVolume(wxCommandEvent& evt)
	{
		m_canvas->ShowVolVis(GetIcons());
	}

	void v4d_Viewer3dFrame::ResizeFrameIfRequired()
	{
		if (GetMenuBar()->IsChecked(SI_AUTO_ADJUST_FRAME_SIZE))
		{
			wxCommandEvent evt;
			OnAdjustFrameSize(evt);
		}
	}

	void v4d_Viewer3dFrame::OnZoomIn(wxCommandEvent& event)
	{
		m_canvas->ZoomIn();
		ResizeFrameIfRequired();
	}

	void v4d_Viewer3dFrame::OnZoomOut(wxCommandEvent& event)
	{
		m_canvas->ZoomOut();
		ResizeFrameIfRequired();
	}

	void v4d_Viewer3dFrame::OnNormalZoom(wxCommandEvent& event)
	{
		m_canvas->NormalZoom();
		ResizeFrameIfRequired();
	}

	void v4d_Viewer3dFrame::OnPreferences(wxCommandEvent& event)
	{
		v4d_CanvasSettings settings;
		m_canvas->GetSettings(settings);

		wxColourData palette;
		v4d_Preferences::GetColourPalette(palette);

		PreferencesDialog dlg(this, &settings, &palette);
		if (dlg.ShowModal() == wxID_OK)
		{
			m_canvas->SetSettings(settings);
			v4d_Preferences::SetColourPalette(palette);
		}
	}

	void v4d_Viewer3dFrame::OnReverseAxis(wxCommandEvent& event)
	{
		m_canvas->ToggleAxisReverse(event.GetId() - SI_REVERSE_X);
	}

	void v4d_Viewer3dFrame::OnMove(wxMoveEvent &evt)
	{
		m_canvas->InfoFrame_Move();
	}

	void v4d_Viewer3dFrame::OnSequenceMenuCmd(wxCommandEvent& evt)
	{
		//dispatches all commands down to the m_canvas to handle them
		wxString msg;
		m_canvas->OnSequenceMenuHandler(evt,msg);
		if (!msg.IsEmpty()) SetInfoStatusText(msg);
	}

	void v4d_Viewer3dFrame::OnUpdateUI(wxUpdateUIEvent &evt)
	{

		switch (evt.GetId())
		{
		case SI_AUTOFOCUS_XY:
			evt.Check(m_canvas->GetContent(pt_XY) == pc_Af);
			break;
		case SI_AUTOFOCUS_XZ:
			evt.Check(m_canvas->GetContent(pt_XZ) == pc_Af);
			break;
		case SI_AUTOFOCUS_YZ:
			evt.Check(m_canvas->GetContent(pt_YZ) == pc_Af);
			break;
		case SI_AF:
			evt.Check(m_canvas->GetContent(pt_All) == pc_Af);
			break;
		case SI_TOGGLE_XY:
			evt.Check(m_canvas->IsDrWinShown(pt_XY));
			break;
		case SI_TOGGLE_XZ:
			evt.Check(m_canvas->IsDrWinShown(pt_XZ));
			break;
		case SI_TOGGLE_YZ:
			evt.Check(m_canvas->IsDrWinShown(pt_YZ));
			break;
		case SI_REVERSE_X:
			evt.Check(m_canvas->IsAxisReversed(0));
			break;
		case SI_REVERSE_Y:
			evt.Check(m_canvas->IsAxisReversed(1));
			break;
		case SI_REVERSE_Z:
			evt.Check(m_canvas->IsAxisReversed(2));
			break;
		case SI_SHOW_WINDOWS_ANNOTATIONS:
			evt.Check(m_canvas->GetInfoFlag());
			break;
		case SI_SHOW_INFO_WIN:
			evt.Check(m_canvas->InfoFrame_IsShown());
			break;
		case SI_REFRESH_ICON:
			RefreshIcon();
			break;
		case SI_SM_PLAY_FORWARD:
			evt.Check(m_canvas->OnSequenceMenuUpdateUI_PlayingForward());
			break;
		case SI_SM_PLAY_BACKWARD:
			evt.Check(m_canvas->OnSequenceMenuUpdateUI_PlayingBackward());
			break;
		case SI_SM_PLAY_LOOP:
			evt.Check(m_canvas->OnSequenceMenuUpdateUI_Loop());
			break;
		case SI_SM_PLAY_OSCILLATION:
			evt.Check(m_canvas->OnSequenceMenuUpdateUI_Oscillation());
			break;
		}
	}

	void v4d_Viewer3dFrame::Reset()
	{
		m_canvas->OnDataChange();
	}

	void v4d_Viewer3dFrame::AddExtensionModule(v4d_ExtensionModule *module, const wxString &menuname, bool delete_extension)
	{
		if (delete_extension)
		{
			wxMenu *menu = module->CreateMenu(m_last_menu_id); //todo menu identifiers
			if (menu != NULL)
				GetMenuBar()->Append(menu, menuname);
		}

		//wxLogMessage(wxString::Format("New extension module of %p added %p", this, module));

		PushEventHandler(module);
		if (module->GetParent() == NULL)
			module->SetModuleParent(this);
		m_delete_extension.push(delete_extension);
	}

	void v4d_Viewer3dFrame::AppendMenu(const wxMenu *menu, const wxString &menu_name)
	{
		wxASSERT(menu);
		wxMenu *new_menu = new wxMenu;
		wxMenuItem *item;

		for (size_t i = 0; i < menu->GetMenuItemCount(); i++)
		{
			item = menu->FindItemByPosition(i);
			new_menu->Append(item->GetId(), item->GetItemLabel(), item->GetHelp(), item->GetKind());
		}

		GetMenuBar()->Append(new_menu, menu_name);
	}

	/******************************************************************************/
	//                            Create Image3dFrame
	/******************************************************************************/
	template <class T>
	v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<T> *images, bool delete_image, wxWindow *parent, 
		wxWindowID id, const wxString& title,
		const wxPoint& pos, const wxSize& size, long style)
	{		
		v4d_Viewer3dFrame *frame
			= new v4d_Viewer3dFrame(parent, id, title, pos, size, style);
		v4d_MappingModule<T> *mapper = v4d_CreateMapperForType<T>(frame,(const i4d::Image4d<T>)*images);
		frame->AddExtensionModule(mapper, "&Mapping");
		frame->SetCanvas(new v4d::v4d_ImageViewer3dCanvas<T>(frame, *mapper, images, delete_image));
		frame->ResizeFrameIfRequired();

		return frame;
	}
	
	// instantiations
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<i3d::GRAY8> *images, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<i3d::GRAY16> *images, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<i3d::RGB> *images, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<i3d::RGB16> *images, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<float> *images, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateImage3dFrame(i4d::Image4d<std::complex<float> > *img, bool,
		wxWindow *, wxWindowID, const wxString&, const wxPoint&, const wxSize&, long);

	/******************************************************************************/
	//                            Create LabeledImage3dFrame
	/******************************************************************************/
	template <class IDATA, class LABEL, class LDATA>
	v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<IDATA> *imgs,
		i4d::LabeledImage4d<LABEL, LDATA> *limgs, bool delete_image, bool delete_lab_image,
		wxWindow *parent, wxWindowID id, const wxString &title,
		const wxPoint &pos, const wxSize &size, long style)
	{
		v4d_Viewer3dFrame *frame = new v4d_Viewer3dFrame(parent, id, title, pos, size, style);
		v4d_OverlayingModule<LABEL, LDATA> *overlayer =
			new v4d_BasicOverlayingModule<LABEL, LDATA>(frame, limgs, delete_lab_image);
		v4d_MappingModule<IDATA> *mapper = v4d_CreateMapperForType<IDATA>(frame, *imgs);

		frame->AddExtensionModule(mapper, "&Mapping");
		frame->AddExtensionModule(overlayer, "&Overlaying");
		frame->SetCanvas(new v4d::v4d_LabeledImageViewer3dCanvas<IDATA, LABEL, LDATA>
			(frame, *mapper, *overlayer, imgs, delete_image));
		frame->ResizeFrameIfRequired();
		return frame;
	}

	// Explicit instantiations	
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY8> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY8> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY8> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::GRAY16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<float> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<float> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<float> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<int> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<int> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<int> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<i3d::RGB16> *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16> *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);

	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<std::complex<float> > *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, bool > *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<std::complex<float> > *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY8 > *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
	template V4D_DLLEXPORT v4d_Viewer3dFrame *v4d_CreateLabeledImage3dFrame(i4d::Image4d<std::complex<float> > *imgs,
		i4d::LabeledImage4d<i3d::GRAY16, i3d::GRAY16 > *limgs, bool, bool, wxWindow *, wxWindowID,
		const wxString&, const wxPoint&, const wxSize&, long);
}
