/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*-*-c++-*-
 * v4d_labeledimagecanvas3d.h
 *
 * Canvas for displaying images together with labeled images
 *
 * Martin Maka (xmaska@fi.muni.cz) 2007
 */
#ifndef V4D_V4D_LABELEDIMAGECANVAS3D_H
#define V4D_V4D_LABELEDIMAGECANVAS3D_H

#include <i3d/regions.h>
#include "types/overlayer.h"
#include "v4d_imagecanvas3d.h"
#include "v4d_image4d.h"

namespace v4d 
{
	/////////////////////////////////////////////////////////////////
	//                                                             //
	//                LabeledImageViewer3dCanvas                   //
	//                                                             //
	/////////////////////////////////////////////////////////////////
	/** Canvas for displaying RGB images together with labeled images. 
	It contains three drawing windows (for each orthogonal projection one). */
	template <class IDATA, class LABEL, class LDATA> 
    class V4D_DLLEXPORT v4d_LabeledImageViewer3dCanvas
        : public v4d_ImageViewer3dCanvas<IDATA>
	{
		size_t m_lab_xy_origbuf;
		size_t m_lab_xz_origbuf;
		size_t m_lab_yz_origbuf;
		size_t m_lab_xy_af;
		size_t m_lab_xz_af;
		size_t m_lab_yz_af;

		bool m_xy_overlay;
		bool m_xz_overlay;
		bool m_yz_overlay;

		v4d_Overlayer<LABEL, LDATA> &m_overlayer;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_origbuf;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_origbuf;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_origbuf;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_af;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_af;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_af;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_rgbbuf;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_rgbbuf;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_rgbbuf;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_bmp;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_bmp;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_bmp;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_dirty;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_dirty;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_dirty;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_remap;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_remap;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_remap;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy_af_dirty;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz_af_dirty;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz_af_dirty;

		using v4d_ImageViewer3dCanvas<IDATA>::m_img;
		using v4d_ImageViewer3dCanvas<IDATA>::m_mapper;

		using v4d_ImageViewer3dCanvas<IDATA>::m_xy;
		using v4d_ImageViewer3dCanvas<IDATA>::m_xz;
		using v4d_ImageViewer3dCanvas<IDATA>::m_yz;
		using v4d_ImageViewer3dCanvas<IDATA>::m_vv;
		using v4d_ImageViewer3dCanvas<IDATA>::GetContent;

		using v4d_ImageViewer3dCanvas<IDATA>::m_info_frame;

	public:
		/** Constructor. It creates an instance of the 3d canvas and
		initialize all its attributes according to the given image or to default
		values if imgs = NULL.  delete_data flag specifies wheather the image data
		is deleted in destructor or not. */
		v4d_LabeledImageViewer3dCanvas(wxWindow *parent, 
			const v4d_Mapper<IDATA> &mapper, v4d_Overlayer<LABEL, LDATA> &overlayer, 
			i4d::Image4d<IDATA> *imgs = NULL, bool delete_data = true, wxWindowID id = -1, 
			const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize);

		/** Destructor. It frees all allocated memory. It can delete also image data
		if requested in constructor or in SetImage function. */
        virtual ~v4d_LabeledImageViewer3dCanvas();

		/** Draw RGB image projection to specified window. */
		virtual void Draw(wxDC &dc, v4d_ProjectionType pt);
        /** Called after overlaying mode or overlay image has changed */
		virtual void OverlayingChanged(v4d_OverlayingEvent &evt);
        /** Called after overlaying mode or overlay image or source image or has changed */
        virtual void OnDataChange();
		/** Called after a time point selection has changed. */
        virtual void OnTimeChange();
        /** Updates the vol3D_Visualization isosurface data*/
        virtual void UpdateIsosurface();

	protected:
		/** Create info frame. */
		virtual void InfoFrame_Create(); 
		/** Update pixel info - intensity, mapped colour, etc. */
		virtual void InfoFrame_UpdatePixelInfo();

	private:
		/** \returns Label colour at given position. */
		void GetLabelInfo(v4d_ProjectionType pt, int x, int y, wxString &col_label) const;
	};
}

#endif
