/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * vol3D_frame.cpp
 *
 * implementation of vol3D_VolVisFrame
 *
 * Marek Vinkler (172521@mail.muni.cz) 2008
 * Ondrej Danek (xdanek2@fi.muni.cz) 2010
 */

#include "v4d_basic.h"
#include "vol3D_frame.h"
#include "wdr/vol3D_dlg_help_wdr.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
#endif

namespace v4d
{
	BEGIN_EVENT_TABLE(vol3D_VolVisFrame, wxFrame)
		EVT_SET_FOCUS(vol3D_VolVisFrame::OnFocus)
		EVT_MENU(ID_VOL3D_HELP, vol3D_VolVisFrame::OnHelp)
        EVT_MENU(ID_VOL3D_HIDE_BOX, vol3D_VolVisFrame::OnShowBox)
        EVT_MENU(ID_VOL3D_HIDE_CLIP, vol3D_VolVisFrame::OnShowClipPlane)
        EVT_MENU(ID_VOL3D_WHITE_BCG, vol3D_VolVisFrame::OnWhiteBcg)
        EVT_MENU(ID_VOL3D_SAVE_VIEW, vol3D_VolVisFrame::OnSaveView)
        EVT_MENU(ID_VOL3D_LOAD_VIEW, vol3D_VolVisFrame::OnLoadView)
		EVT_MENU_RANGE(ID_VOL3D_PX, ID_VOL3D_NZ, vol3D_VolVisFrame::OnView)
		EVT_CLOSE(vol3D_VolVisFrame::OnClose)
	END_EVENT_TABLE()

	vol3D_VolVisFrame::vol3D_VolVisFrame(wxWindow *parent, wxWindowID id, const wxString& title, const wxPoint& pos,
		const wxSize& size, long style):
	wxFrame(parent, id, title, pos, size, style), _help(NULL)
	{
		SetMenuBar(vol3D_VolVisFrameMenuFunc());
		CreateStatusBar(2);

		_canvas = new vol3D_VolVisCanvas(GetStatusBar(), this, wxID_ANY, wxPoint(0,0), this->GetClientSize());
	}

	vol3D_VolVisFrame::~vol3D_VolVisFrame()
	{
		if(_help)
			delete _help;
	}

	void vol3D_VolVisFrame::OnFocus(wxFocusEvent& evt)
	{
		_canvas->SetFocus();
	}

	void vol3D_VolVisFrame::OnHelp(wxCommandEvent &evt)
	{
		if (!_help)
		{
			_help = new wxDialog(this, wxID_ANY, "Volume Visualization Help");
			vol3D_Dlg_HelpFunc(_help);
		}

		_help -> Show();
		_help -> SetFocus();
	}

	void vol3D_VolVisFrame::OnView(wxCommandEvent &evt)
	{
		float distance, ax, ay, az, ux, uy, uz;

		distance = -5.0f;
		switch(evt.GetId())
		{
		case ID_VOL3D_PX:
			ax = 1.0f;
			ay = 0.0f;
			az = 0.0f;
			ux = 0.0f;
			uy = 1.0f;
			uz = 0.0f;
			break;
		case ID_VOL3D_NX:
			ax = -1.0f;
			ay = 0.0f;
			az = 0.0f;
			ux = 0.0f;
			uy = 1.0f;
			uz = 0.0f;
			break;
		case ID_VOL3D_PY:
			ax = 0.0f;
			ay = 1.0f;
			az = 0.0f;
			ux = 0.0f;
			uy = 0.0f;
			uz = -1.0f;
			break;
		case ID_VOL3D_NY:
			ax = 0.0f;
			ay = -1.0f;
			az = 0.0f;
			ux = 0.0f;
			uy = 0.0f;
			uz = -1.0f;
			break;
		case ID_VOL3D_PZ:
			ax = 0.0f;
			ay = 0.0f;
			az = 1.0f;
			ux = 0.0f;
			uy = 1.0f;
			uz = 0.0f;
			break;
		case ID_VOL3D_NZ:
			ax = 0.0f;
			ay = 0.0f;
			az = -1.0f;
			ux = 0.0f;
			uy = 1.0f;
			uz = 0.0f;
			break;
		default:
			return;
		}

		_canvas->SetView(distance, ax, ay, az, ux, uy, uz);
	}

    void vol3D_VolVisFrame::OnWhiteBcg(wxCommandEvent &evt)
    {
        if (evt.IsChecked())
        {
            _canvas->SetBcgColour(1.0f, 1.0f, 1.0f);
        }
        else
        {
            _canvas->SetBcgColour(0.0f, 0.0f, 0.0f);
        }
    }

    void vol3D_VolVisFrame::OnShowBox(wxCommandEvent &evt)
    {
        _canvas->ShowBox(!evt.IsChecked());
    }

    void vol3D_VolVisFrame::OnShowClipPlane(wxCommandEvent &evt)
    {
        _canvas->ShowClipPlane(!evt.IsChecked());
    }

    void vol3D_VolVisFrame::OnSaveView(wxCommandEvent &evt)
    {
        _canvas->SaveViewMatrix();
    }

    void vol3D_VolVisFrame::OnLoadView(wxCommandEvent &evt)
    {
        _canvas->LoadViewMatrix();
    }

	void vol3D_VolVisFrame::OnClose(wxCloseEvent& evt)
	{
		if(!evt.CanVeto()) //destroy the frame if we have to
		{
			Destroy();
		}
		else //hide it otherwise
		{
			if(_help)
			{
				delete _help;
				_help = NULL;
			}
			Show(false);
			evt.Veto();
		}
	}

	vol3D_VolVisCanvas* vol3D_VolVisFrame::GetVolVisCanvas()
	{
		return _canvas;
	}

} //namespace v4d
