/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*=================================================================
 *
 * call a wxapplication
 *
 *=================================================================*/
/* $Revision: 1.2 $ */
#include <math.h>
#include <string.h>
#include "mex.h"
#include "dipview3d.h"

extern const char *err_msg;

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray* prhs[] )
     
{ 
	if (nlhs > 1 || nrhs != 1 || !mxIsClass(prhs[0], "dip_image"))
		mexErrMsgTxt("This function requires one input dip_image and one output argument.");

	mxArray** first_arg = const_cast<mxArray**>(&prhs[0]);

	// check wheather the image is a color image
	mxArray *lhs;
	mexCallMATLAB(1, &lhs, 1, first_arg, "iscolor");
	bool is_color = mxIsLogicalScalarTrue(lhs);
	if (is_color)
	{
		mexErrMsgTxt("Colour images are not supported yet.");
	}

	// get info about datatype
	mxArray *dtarr;
	const int buflen = 20;
	char buf[buflen];
	mexCallMATLAB(1, &dtarr, 1,  first_arg, "datatype");
	mxGetString(dtarr, buf, buflen);
	mexPrintf("Data type: %s (size %d)\n", buf, mxGetElementSize(dtarr));


	// set correct data type
	const char *dipimage_type;
	ViewerDataType viewer_type;

	if (strcmp(buf, "uint8") == 0)
	{
		dipimage_type = "uint8";
		viewer_type = VDT_GRAY8;
	}
	else if (strcmp(buf, "uint16") == 0)
	{
		dipimage_type = "uint16";
		viewer_type = VDT_GRAY16;
	}
	else if (strcmp(buf, "sfloat") == 0)
	{
		dipimage_type = "single";
		viewer_type = VDT_FLOAT;
	}
	else
		mexErrMsgTxt("This function works only for uint8, uint16 or sfloat images");

	// get size information
	mxArray *szarr;
	mexCallMATLAB(1, &szarr, 1,  first_arg, "size");
	double *sz = (double *) mxGetPr(szarr);

	// check dimensions
	if (mxGetNumberOfElements(szarr) != 3)
		mexErrMsgTxt("Only 3D images are currently supported");

	mexPrintf("Image size: %g %g %g\n", sz[0], sz[1], sz[2]);

	// get pointer to data
	mexCallMATLAB(1, &lhs, 1,  first_arg, dipimage_type);
	void *data = mxGetData(lhs);

	// resolution 
	float res[3];
	res[0] = 10;
	res[1] = 10;
	res[2] = 10;

	switch (Run3DViewer(sz, res, viewer_type, data))
	{
	case 0:
		mexPrintf("Initialization of wxWidgets successful\n");
		break;
	case -1:
		mexPrintf("Cannot create a new thread!\n");
		break;
	case -2:
		mexPrintf("Cannot terminate the running thread!\n");
		break;
	case -3:
		mexPrintf("Unsupported data type!\n");
		break;
	default:
		mexPrintf("Unknown return code\n");
	}
    return;
    
}


