/*
 *  v4dwins
 *
 *  Copyright (C) 2006 - 2009   Centre for Biomedical Image Analysis (CBIA)
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* vol3D_frame.h
 *
 * Volume visualization frame exploiting OpenGL
 *
 * Radeon Volume Visualization
 * '+'     - Increase number of slices
 * '-'     - Decrease number of slices
 * <SPACE> - Switch between moving clip plane and object
 * <UP>    - Increase the opacity of slices
 * <DOWN>  - Decrease the opacity of slices
 * <LEFT>  - Decrease minimum density displayed
 * <RIGHT> - Increase minimum density displayed
 * <ESC>   - Quit
 *
 * Marek Vinkler (172521@mail.muni.cz) 2008
 */

#ifndef V4D_VOL3D_FRAME_H
#define V4D_VOL3D_FRAME_H


#include <wx/frame.h>
#include <wx/dialog.h>
#include "vol3D_canvas.h"

namespace v4d
{
	/** Volume visualization frame exploiting OpenGL. */
	class vol3D_VolVisFrame : public wxFrame
	{
	private:
		vol3D_VolVisCanvas *_canvas;
		wxDialog *_help;

	public:
		/** Constructor. */
		vol3D_VolVisFrame(wxWindow *parent, wxWindowID id = wxID_ANY, const wxString& title = "VolVis frame",
			const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
		/** Destructor. */
		virtual ~vol3D_VolVisFrame();

		void OnFocus(wxFocusEvent &evt);
		void OnHelp(wxCommandEvent &evt);
		void OnClose(wxCloseEvent &evt);

		vol3D_VolVisCanvas* GetVolVisCanvas();

    private:
		void OnView(wxCommandEvent &evt);
        void OnShowBox(wxCommandEvent &evt);
        void OnShowClipPlane(wxCommandEvent &evt);
        void OnWhiteBcg(wxCommandEvent &evt);
        void OnSaveView(wxCommandEvent &evt);
        void OnLoadView(wxCommandEvent &evt);

		DECLARE_EVENT_TABLE()
	};

} // namespace v4d

#endif // __VOL3D_FRAME_H__
